from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site


def project_settings(request):
    return {
        'PLATFORM_NAME': get_current_site(request).name,
        'DEBUG': settings.DEBUG
    }



from django import template

register = template.Library()


@register.simple_tag
def is_selected(value, arr):
    if str(value) in arr:
        return 'selected'
    return ''


@register.simple_tag
def show_restrict(rules, category):
    if rules:
        if str(category.id) in rules:
            if rules[str(category.id)] == {}:
                return ('<a href="#" class="text-danger activate-detailed">'
                        'Restrict further</a>')

    return ''


@register.filter(name='is_in')
def is_in(d, key_name):
    if d is not None:
        if str(key_name) in d:
            return True

    return False


@register.inclusion_tag('users/data_fields_rules.html')
def show_fields(filters, category):
    if filters and str(category.id) in filters:
        cat_rules = filters.get(str(category.id))

        context = {
            'min_date': cat_rules.pop('min_date', None),
            'max_date': cat_rules.pop('max_date', None)
        }

        context['fields'] = [{
                'category_id': category.id,
                'field': category.fields.get_subclass(key=key),
                'rule': cat_rules[key]
            } for key in cat_rules]

        return context


@register.filter(name='minval')
def minval(d):
    if d is not None:
        if d.get('minval'):
            return d.get('minval')

    return ''


@register.filter(name='maxval')
def maxval(d):
    if d is not None:
        if d.get('maxval'):
            return d.get('maxval')

    return ''



import requests
import tempfile
from django.core import files

from django.core.exceptions import PermissionDenied, ValidationError

from easy_thumbnails.files import get_thumbnailer

from rest_framework import serializers
from rest_framework_gis import serializers as geoserializers

from rest_framework.serializers import BaseSerializer

from geokey.categories.serializer import CategorySerializer
from geokey.categories.models import Category
from geokey.users.serializers import UserSerializer

from .models import Observation, Comment
from .models import Location
from .models import MediaFile, ImageFile, VideoFile


class LocationSerializer(geoserializers.GeoFeatureModelSerializer):
    """
    Serialiser for geokey.contribtions.models.Location
    """
    class Meta:
        model = Location
        geo_field = 'geometry'
        fields = ('id', 'name', 'description', 'status', 'created_at')
        write_only_fields = ('status',)


class LocationContributionSerializer(serializers.ModelSerializer):
    """
    Serialiser for `Location`; to be used within `ContributionSerializer`.
    """
    class Meta:
        model = Location
        fields = ('id', 'name', 'description', 'status', 'created_at',
                  'geometry', 'private', 'private_for_project')
        write_only_fields = ('status', 'private', 'private_for_project')

    def create(self, validated_data):
        """
        Creates a new contribution from `validated_data`

        Parameter
        ---------
        validated_data : dict
            Input data after validation

        Returns
        -------
        Location
        """
        validated_data['creator'] = self.context.get('user')
        return super(
            LocationContributionSerializer,
            self
        ).create(validated_data)


class ContributionSerializer(BaseSerializer):
    """
    Serialiser for geokey.contribtions.models.Observations. This is a custom
    serialiser, not a standard ModelSerializer
    """
    @classmethod
    def many_init(cls, *args, **kwargs):
        """
        Is called when many=True property is set when instantiating the
        serialiser.
        """
        kwargs['context']['many'] = True
        return super(ContributionSerializer, cls).many_init(*args, **kwargs)

    def validate_category(self, project, category_id):
        """
        Validates if the category can be used with the project

        Parameters
        ----------
        project : geokey.projects.models.Project
            Project that the category is used for
        category_id : int
            identifies the category in the database

        Returns
        -------
        geokey.categories.models.Category
            The valid category
        """
        errors = []
        category = None
        try:
            category = project.categories.get(pk=category_id)
            if category.status == 'inactive':
                errors.append('The category can not be used because it is '
                              'inactive.')
            else:
                self._validated_data['meta']['category'] = category
        except Category.DoesNotExist:
            errors.append('The category can not be used with the project '
                          'or does not exist.')

        if errors:
            self._errors['category'] = errors

        return category

    def replace_null(self, properties):
        """
        Replaces all empty str or unicode values with None and returns the
        properies dict

        Parameter
        ---------
        properties : dict
            Contribution properties

        Returns
        -------
        dict
            Contribution properties with replaced null values

        """
        for key, value in properties.iteritems():
            if isinstance(value, (str, unicode)) and len(value) == 0:
                properties[key] = None

        return properties

    def validate_properties(self, properties, category=None, status=None):
        """
        Validates the properties and adds error messages to self._errors

        Parameter
        ---------
        properties : dict
            Contribution properties
        category : geokey.categories.models.Category
            Category the properties are validated against
        status : str
            Status for the contribution
        """
        errors = []

        if self.instance:
            status = status or self.instance.status
            if self.instance.properties:
                update = self.instance.properties.copy()
                update.update(properties)
                properties = update
        else:
            status = status or category.default_status

        properties = self.replace_null(properties)
        try:
            if status == 'draft':
                Observation.validate_partial(category, properties)
            else:
                Observation.validate_full(category, properties)
        except ValidationError, e:
            errors.append(e)

        self._validated_data['properties'] = properties
        self._validated_data['meta']['status'] = status

        if errors:
            self._errors['properties'] = errors

    def validate_location(self, project, location_id):
        """
        Validates if the location can be used with the project

        Parameters
        ----------
        project : geokey.projects.models.Project
            Project that the category is used for
        location_id : int
            identifies the location in the database
        """
        errors = []
        self.location = None

        try:
            if location_id is not None:
                self.location = Location.objects.get_single(
                    self.context.get('user'),
                    project.id,
                    location_id
                )
        except PermissionDenied, error:
            errors.append(error)
        except Location.DoesNotExist, error:
            errors.append(error)

        if errors:
            self._errors['location'] = errors

    def is_valid(self, raise_exception=False):
        """
        Checks if the contribution that is deserialised is valid. Validates
        location, category and properties.

        Parameter
        ---------
        raise_exception : Boolean
            indicates if an exeption should be raised if the data is invalid.
            If set to false, this method will return False if the data is
            invalid.

        Returns
        -------
        Boolean
            indicating if data is valid

        Raises
        ------
        ValidationError
            If data is invalid. Exception is raised when raise_exception is set
            tp True.
        """
        self._errors = {}
        self._validated_data = self.initial_data

        project = self.context.get('project')
        meta = self.initial_data.get('meta')

        if meta is None:
            self._validated_data['meta'] = dict()

        # Validate location
        location_id = None
        if self.initial_data.get('location') is not None:
            location_id = self.initial_data.get('location').get('id')
        self.validate_location(project, location_id)

        # Validate category
        category = None
        if self.instance is None and meta is not None:
            category = self.validate_category(project, meta.get('category'))
        else:
            category = self.instance.category
        self._validated_data['meta']['category'] = category

        # Validatie properties
        properties = self.initial_data.get('properties') or {}
        status = None
        if meta is not None:
            status = meta.get('status', None)
        if properties is not None and category is not None:
            self.validate_properties(
                properties,
                category=category,
                status=status
            )

        # raise the exception
        if self._errors and raise_exception:
            raise ValidationError(self._errors)

        return not bool(self._errors)

    def create(self, validated_data):
        """
        Creates a new observation and returns the instance.

        Parameter
        ---------
        validated_data : dict
            the data dict after validation

        Returns
        -------
        geokey.contributions.models.Observation
            The instance created
        """
        project = self.context.get('project')
        meta = validated_data.pop('meta')

        location_serializer = LocationContributionSerializer(
            self.location,
            data=validated_data.pop('location', None),
            context=self.context
        )
        if location_serializer.is_valid():
            location_serializer.save()

        self.instance = Observation.create(
            properties=validated_data.get('properties'),
            creator=self.context.get('user'),
            location=location_serializer.instance,
            project=project,
            category=meta.get('category'),
            status=meta.pop('status', None)
        )

        return self.instance

    def update(self, instance, validated_data):
        """
        Updates an existing observation and returns the instance.

        Parameter
        ---------
        instance : geokey.contributions.models.Observation
            the instance to be updated
        validated_data : dict
            the data dict after validation

        Returns
        -------
        geokey.contributions.models.Observation
            The instance update
        """
        meta = validated_data.get('meta')
        status = None
        if meta is not None:
            status = meta.get('status', None)

        location_serializer = LocationContributionSerializer(
            instance.location,
            data=validated_data.pop('location', {}),
            context=self.context,
            partial=True
        )
        if location_serializer.is_valid():
            location_serializer.save()

        return instance.update(
            properties=validated_data.get('properties'),
            updator=self.context.get('user'),
            status=status
        )

    def get_display_field(self, obj):
        """
        Returns a native representation of the display_field property.

        Parameter
        ---------
        obj : geokey.contributions.models.Observation
            The instance that is serialised

        Returns
        -------
        dict
            serialised display_field; e.g.
            {
                'key': 'field_key',
                'value': 'The value of the field'
            }
        """
        if obj.display_field is not None:
            display_field = obj.display_field.split(':', 1)
            value = display_field[1] if display_field[1] != 'None' else None
            return {
                'key': display_field[0],
                'value': value
            }
        else:
            return None

    def get_search_result(self, obj, q):
        """
        Returns all fields which values have matched a search query

        Parameter
        ---------
        obj : geokey.contributions.models.Observation
            The instance that is serialised
        q : str
            The query string of the search

        Return
        ------
        dict
            the field that matched the query, e.g.
            {
                'field_key_1': 'value 1',
                'field_key_2': 'value 2',
            }
        """
        search_matches = {}

        matcher = obj.search_matches.split('#####')

        for field in matcher:
            match = field.split(':', 1)
            if q.lower() in match[1].lower():
                search_matches[match[0]] = match[1]

        return search_matches

    def to_representation(self, obj):
        """
        Returns the native representation of a contribution

        Parameter
        ---------
        obj : geokey.contributions.models.Observation
            The instance that is serialised

        Returns
        -------
        dict
            Native represenation of the Contribution
        """
        location = obj.location

        isowner = False
        if not self.context.get('user').is_anonymous():
            isowner = obj.creator == self.context.get('user')

        updator = None
        if obj.updator is not None:
            updator = {
                'id': obj.updator.id,
                'display_name': obj.updator.display_name
            }

        feature = {
            'id': obj.id,
            'properties': obj.properties,
            'display_field': self.get_display_field(obj),
            'meta': {
                'status': obj.status,
                'creator': {
                    'id': obj.creator.id,
                    'display_name': obj.creator.display_name
                },
                'updator': updator,
                'created_at': str(obj.created_at),
                'updated_at': str(obj.updated_at),
                'version': obj.version,
                'isowner': isowner,
                'num_comments': obj.comments.count(),
                'num_media': obj.files_attached.count()
            },
            'location': {
                'id': location.id,
                'name': location.name,
                'description': location.description,
                'geometry': location.geometry.geojson
            }
        }

        if self.context.get('many'):
            cat = obj.category
            feature['meta']['category'] = {
                'id': cat.id,
                'name': cat.name,
                'description': cat.description,
                'symbol': cat.symbol.url if cat.symbol else None,
                'colour': cat.colour
            }

            q = self.context.get('search')
            if q is not None:
                feature['search_matches'] = self.get_search_result(obj, q)
        else:
            category_serializer = CategorySerializer(
                obj.category, context=self.context)
            feature['meta']['category'] = category_serializer.data

            comment_serializer = CommentSerializer(
                obj.comments.filter(respondsto=None),
                many=True,
                context=self.context
            )
            feature['comments'] = comment_serializer.data

            review_serializer = CommentSerializer(
                obj.comments.filter(review_status='open'),
                many=True,
                context=self.context
            )
            feature['review_comments'] = review_serializer.data

            file_serializer = FileSerializer(
                obj.files_attached.all(),
                many=True,
                context=self.context
            )
            feature['media'] = file_serializer.data

        return feature


class CommentSerializer(serializers.ModelSerializer):
    """
    Serialiser for geokey.contributions.models.Comment
    """
    creator = UserSerializer(fields=('id', 'display_name'), read_only=True)
    isowner = serializers.SerializerMethodField()

    class Meta:
        model = Comment
        fields = ('id', 'respondsto', 'created_at', 'text', 'isowner',
                  'creator', 'review_status')
        read_only = ('id', 'respondsto', 'created_at')

    def to_representation(self, obj):
        """
        Returns native represenation of the Comment. Adds reponses to comment

        Parameter
        ---------
        obj : geokey.contributions.models.Comment
            The instance that is serialised

        Returns
        -------
        dict
            Native represenation of the Comment

        """
        native = super(CommentSerializer, self).to_representation(obj)
        native['responses'] = CommentSerializer(
            obj.responses.all(),
            many=True,
            context=self.context
        ).data

        return native

    def get_isowner(self, comment):
        """
        Returns True if the user serialising the Comment has created the
        comment

        Parameter
        ---------
        comment : geokey.contributions.models.Comment
            The instance that is serialised

        Returns
        -------
        Boolean
            indicating of user is creator of comment
        """
        if not self.context.get('user').is_anonymous():
            return comment.creator == self.context.get('user')
        else:
            return False


class FileSerializer(serializers.ModelSerializer):
    """
    Serialiser for geokey.contributions.models.MediaFile instances
    """
    creator = UserSerializer(fields=('id', 'display_name'))
    isowner = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()
    file_type = serializers.SerializerMethodField()
    thumbnail_url = serializers.SerializerMethodField()

    class Meta:
        model = MediaFile
        fields = (
            'id', 'name', 'description', 'created_at', 'creator', 'isowner',
            'url', 'thumbnail_url', 'file_type'
        )

    def get_file_type(self, obj):
        """
        Returns the type of the MediaFile

        Parameter
        ---------
        obj : geokey.contributions.models.MediaFile
            The instance that is serialised

        Returns
        -------
        str
            The type of the, e.g. 'ImageFile'
        """
        return obj.type_name

    def get_isowner(self, obj):
        """
        Returns `True` if the user provided in the serializer context is the
        creator of this file

        Parameter
        ---------
        obj : geokey.contributions.models.MediaFile
            The instance that is serialised

        Returns
        -------
        Boolean
            indicating if user created the file
        """
        if not self.context.get('user').is_anonymous():
            return obj.creator == self.context.get('user')
        else:
            return False

    def get_url(self, obj):
        """
        Return the url to access this file based on its file type

        Parameter
        ---------
        obj : geokey.contributions.models.MediaFile
            The instance that is serialised

        Returns
        -------
        str
            The URL to embed the file on client side
        """
        if isinstance(obj, ImageFile):
            return obj.image.url
        elif isinstance(obj, VideoFile):
            return obj.youtube_link

    def _get_thumb(self, image, size=(300, 300)):
        """
        Returns the thumbnail of the media file base on the size privoded

        Parameter
        ---------
        image : Image
            The image to be thumbnailed
        size : tuple
            width and height of the thumbnail, defaults to 300 by 300

        Returns
        -------
        Image
            The thumbnail

        """
        thumbnailer = get_thumbnailer(image)
        thumb = thumbnailer.get_thumbnail({
            'crop': True,
            'size': size
        })
        return thumb

    def get_thumbnail_url(self, obj):
        """
        Creates and returns a thumbnail for the MediaFile object

        Parameter
        ---------
        obj : geokey.contributions.models.MediaFile
            The instance that is serialised

        Returns
        -------
        str
            The url to embed thumbnails on client side
        """
        if isinstance(obj, ImageFile):
            # Some of the imported image files in the original community maps
            # seem to be broken. The error thrown when the image can not be
            # read is caught here.
            try:
                return self._get_thumb(obj.image).url
            except IOError:
                return ''

        elif isinstance(obj, VideoFile):
            if obj.thumbnail:
                # thumbnail has been downloaded, return the link
                return self._get_thumb(obj.thumbnail).url

            request = requests.get(
                'http://img.youtube.com/vi/%s/0.jpg' % obj.youtube_id,
                stream=True
            )

            if request.status_code != requests.codes.ok:
                # Image not found, return placeholder thumbnail
                return '/static/img/play.png'

            lf = tempfile.NamedTemporaryFile()
            # Read the streamed image in sections
            for block in request.iter_content(1024 * 8):

                # If no more file then stop
                if not block:
                    break

                # Write image block to temporary file
                lf.write(block)

            file_name = obj.youtube_id + '.jpg'
            obj.thumbnail.save(file_name, files.File(lf))

            from PIL import Image

            w, h = Image.open(obj.thumbnail).size

            thumb = self._get_thumb(obj.thumbnail, size=(h, h))
            obj.thumbnail.save(file_name, thumb)

            return self._get_thumb(obj.thumbnail).url



from django.db import models
from django.db.models import Q
from aggregate_if import Count
from django.core.exceptions import PermissionDenied

from .base import STATUS


class ProjectQuerySet(models.query.QuerySet):
    """
    Custom QuerySet for geokey.projects.models.Project
    """
    def for_user(self, user):
        """
        Returns the projects for the user.

        For anonymous users:
            Returns all projects that are public and have at least one public
            data grouping.

        For authenticated users:
            Returns all projects that
                - are public
                - all projects the user can contribute or moderate

        Parameter
        ---------
        user : geokey.users.models.User
            User projects are queried for

        Return
        ------
        django.db.models.query.QuerySet
            List of geokey.projects.models.Project
        """
        if user.is_anonymous():
            return self.filter(
                    Q(status=STATUS.active) & Q(isprivate=False)
                ).distinct()
        else:
            projects = self.filter(
                Q(admins=user) |
                (
                    Q(status=STATUS.active) &
                    (
                        Q(isprivate=False) |
                        Q(usergroups__users=user)
                    )
                )
            ).distinct()
            return projects


class ProjectManager(models.Manager):
    """
    Custom Manager for geokey.projects.models.Project
    """
    use_for_related_fields = True

    def get_queryset(self):
        """
        Returns the QuerySet excluding deleted projects

        Returns
        -------
        django.db.models.query.QuerySet
            List of geokey.projects.models.Project
        """
        return ProjectQuerySet(self.model).exclude(status=STATUS.deleted)

    def get_list(self, user):
        """
        Returns a list of all projects the user is allowed to access

        Parameter
        ---------
        user : geokey.users.models.User
            User projects are queried for

        Return
        ------
        django.db.models.query.QuerySet
            List of geokey.projects.models.Project
        """
        return self.get_queryset().for_user(user)

    def get_single(self, user, project_id):
        """
        Returns a single project or raises PermissionDenied if the user is not
        allowed to access the project.

        Parameter
        ---------
        user : geokey.users.models.User
            User projects are queried for
        project_id : int
            identifies the project in the database

        Return
        ------
        geokey.projects.models.Project
        """
        return self.get_list(user).get(pk=project_id)

    def as_admin(self, user, project_id):
        """
        Returns the project if the user is member of the administrators group.

        Parameter
        ---------
        user : geokey.users.models.User
            User projects are queried for
        project_id : int
            identifies the project in the database

        Return
        ------
        geokey.projects.models.Project

        Raises
        ------
        PermissionDenied
            If user is not administrator of the project
        """
        if user.is_superuser:
            return self.get(pk=project_id)

        project = self.get_single(user, project_id)
        if project.is_admin(user):
            return project
        else:
            raise PermissionDenied('You are not member of the administrators '
                                   'group of this project and therefore not '
                                   'allowed to alter the settings of the '
                                   'project')

    def as_contributor(self, user, project_id):
        """
        Returns the project if the user is eligable to contribute data to the
        project.

        Parameter
        ---------
        user : geokey.users.models.User
            User projects are queried for
        project_id : int
            identifies the project in the database

        Return
        ------
        geokey.projects.models.Project

        Raises
        ------
        PermissionDenied
            If user is not administrator of the project
        """
        project = self.get_single(user, project_id)
        if project.can_contribute(user):
            return project
        else:
            raise PermissionDenied('You are not eligable to contribute data '
                                   'to this project')



import json
import os
import glob

from PIL import Image
from StringIO import StringIO

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser
from django.core.files.base import ContentFile
from django.core.exceptions import PermissionDenied
from django.conf import settings

from nose.tools import raises
from rest_framework.test import APIRequestFactory, force_authenticate
from rest_framework.renderers import JSONRenderer

from geokey.core.exceptions import MalformedRequestData
from geokey.projects.tests.model_factories import UserF, ProjectF
from geokey.contributions.models import MediaFile

from geokey.contributions.views.media import (
    MediaFileListAbstractAPIView, AllContributionsMediaAPIView,
    MediaFileSingleAbstractView, AllContributionsSingleMediaApiView
)

from ..model_factories import ObservationFactory
from .model_factories import ImageFileFactory, get_image


class MediaFileAbstractListAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project}
        )

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/**/*'
        ))
        for f in files:
            os.remove(f)

    def render(self, response):
        response.accepted_renderer = JSONRenderer()
        response.accepted_media_type = 'application/json'
        response.renderer_context = {'blah': 'blubb'}
        return response.render()

    def test_get_list_and_respond(self):
        ImageFileFactory.create_batch(5, **{'contribution': self.contribution})

        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        request = self.factory.get(url)
        view = MediaFileListAbstractAPIView()
        view.request = request

        response = self.render(
            view.get_list_and_respond(self.admin, self.contribution)
        )
        self.assertEqual(len(json.loads(response.content)), 5)

    def test_create_and_respond(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        data = {
            'name': 'A test image',
            'description': 'Test image description',
            'file': get_image()
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaFileListAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(self.admin, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )

    @raises(MalformedRequestData)
    def test_create_and_respond_without_file(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        data = {
            'name': 'A test image',
            'description': 'Test image description'
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaFileListAbstractAPIView()
        view.request = request

        view.create_and_respond(self.admin, self.contribution)

    @raises(MalformedRequestData)
    def test_create_and_respond_without_name(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        data = {
            'description': 'Test image description',
            'file': get_image()
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaFileListAbstractAPIView()
        view.request = request

        view.create_and_respond(self.admin, self.contribution)

    # def test_create_video_and_respond(self):
    #     url = reverse(
    #         'api:project_media',
    #         kwargs={
    #             'project_id': self.project.id,
    #             'contribution_id': self.contribution.id
    #         }
    #     )
    #
    #     video = open(
    #         '/home/oroick/opencomap/contributions/tests/media/video.MOV'
    #     )
    #
    #     data = {
    #         'name': 'A test image',
    #         'description': 'Test image description',
    #         'file': video
    #     }
    #
    #     request = self.factory.post(url, data)
    #     request.user = self.admin
    #     view = MediaFileListAbstractAPIView()
    #     view.request = request
    #
    #     response = self.render(
    #         view.create_and_respond(self.admin, self.contribution)
    #     )
    #
    #     response_json = json.loads(response.content)
    #     self.assertEqual(
    #         response_json.get('name'),
    #         data.get('name')
    #     )
    #     self.assertEqual(
    #         response_json.get('description'),
    #         data.get('description')
    #     )
    #     self.assertEqual(
    #         response_json.get('creator').get('display_name'),
    #         request.user.display_name
    #     )


class MediaFileSingleAbstractViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project, 'creator': self.creator}
        )

        self.image_file = ImageFileFactory.create(
            **{'contribution': self.contribution, 'creator': self.creator}
        )

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def render(self, response):
        response.accepted_renderer = JSONRenderer()
        response.accepted_media_type = 'application/json'
        response.renderer_context = {'blah': 'blubb'}
        return response.render()

    def test_get_and_respond(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.get(url)
        view = MediaFileSingleAbstractView()
        view.request = request

        response = self.render(
            view.get_and_respond(
                self.admin,
                self.image_file
            )
        )
        response_json = json.loads(response.content)
        self.assertEqual(response_json.get('id'), self.image_file.id)

    @raises(MediaFile.DoesNotExist)
    def test_delete_and_respond_with_admin(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.delete(url)
        view = MediaFileSingleAbstractView()
        view.request = request

        view.delete_and_respond(self.admin, self.image_file)
        MediaFile.objects.get(pk=self.image_file.id)

    @raises(MediaFile.DoesNotExist)
    def test_delete_and_respond_with_contributor(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.delete(url)
        view = MediaFileSingleAbstractView()
        view.request = request

        view.delete_and_respond(self.creator, self.image_file)
        MediaFile.objects.get(pk=self.image_file.id)

    @raises(PermissionDenied)
    def test_delete_and_respond_with_some_dude(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.delete(url)
        view = MediaFileSingleAbstractView()
        view.request = request

        view.delete_and_respond(UserF.create(), self.image_file)


class AllContributionsMediaAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.viewer = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project, 'creator': self.creator}
        )

        ImageFileFactory.create_batch(5, **{'contribution': self.contribution})

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def get(self, user):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        request = self.factory.get(url)
        force_authenticate(request, user)
        view = AllContributionsMediaAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id
        ).render()

    def post(self, user, data=None):
        if data is None:
            data = {
                'name': 'A test image',
                'description': 'Test image description',
                'file': get_image()
            }

        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        request = self.factory.post(url, data)
        force_authenticate(request, user)
        view = AllContributionsMediaAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id
        ).render()

    def test_get_images_with_admin(self):
        response = self.get(self.admin)
        self.assertEqual(response.status_code, 200)

    def test_get_images_with_contributor(self):
        response = self.get(self.creator)
        self.assertEqual(response.status_code, 200)

    def test_get_images_with_some_dude(self):
        response = self.get(UserF.create())
        self.assertEqual(response.status_code, 404)

    def test_get_images_with_anonymous(self):
        response = self.get(AnonymousUser())
        self.assertEqual(response.status_code, 404)

    def test_upload_image_with_admin(self):
        response = self.post(self.admin)
        self.assertEqual(response.status_code, 201)

    def test_upload_image_with_contributor(self):
        response = self.post(self.creator)
        self.assertEqual(response.status_code, 201)

    def test_upload_image_with_some_dude(self):
        response = self.post(UserF.create())
        self.assertEqual(response.status_code, 404)

    def test_upload_image_with_anonymous(self):
        response = self.post(AnonymousUser())
        self.assertEqual(response.status_code, 404)

    def test_post_images_with_anonymous_to_public(self):
        self.project.isprivate = False
        self.project.everyone_contributes = 'true'
        self.project.save()

        response = self.post(AnonymousUser())
        self.assertEqual(response.status_code, 201)

    def test_upload_unsupported_file_format(self):
        xyz_file = StringIO()
        xyz = Image.new('RGBA', size=(50, 50), color=(256, 0, 0))
        xyz.save(xyz_file, 'png')
        xyz_file.seek(0)

        data = {
            'name': 'A test image',
            'description': 'Test image description',
            'file': ContentFile(xyz_file.read(), 'test.xyz')
        }

        response = self.post(self.admin, data=data)
        self.assertEqual(response.status_code, 400)

    def test_upload_with_loooooong_filename(self):
        data = {
            'name': 'A test image ',
            'description': 'Test image description',
            'file': get_image(file_name='One two three four six seven eight '
                                        'nine ten eleven twelve thirteen '
                                        'fourteen fifteen.png')
        }

        response = self.post(self.admin, data=data)
        self.assertEqual(response.status_code, 201)


class AllContributionsSingleMediaApiViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.viewer = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project, 'creator': self.creator}
        )

        self.image_file = ImageFileFactory.create(
            **{'contribution': self.contribution, 'creator': self.creator}
        )

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def get(self, user):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.get(url)
        force_authenticate(request, user)
        view = AllContributionsSingleMediaApiView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id,
            file_id=self.image_file.id
        ).render()

    def delete(self, user, image_id=None):
        if image_id is None:
            image_id = self.image_file.id

        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': image_id
            }
        )

        request = self.factory.delete(url)
        force_authenticate(request, user)
        view = AllContributionsSingleMediaApiView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id,
            file_id=image_id
        ).render()

    def test_get_image_with_admin(self):
        response = self.get(self.admin)
        self.assertEqual(response.status_code, 200)

    def test_get_image_with_contributor(self):
        response = self.get(self.creator)
        self.assertEqual(response.status_code, 200)

    def test_get_image_with_some_dude(self):
        response = self.get(UserF.create())
        self.assertEqual(response.status_code, 404)

    def test_get_image_with_anonymous(self):
        response = self.get(AnonymousUser())
        self.assertEqual(response.status_code, 404)

    def test_delete_image_with_admin(self):
        response = self.delete(self.admin)
        self.assertEqual(response.status_code, 204)

    def test_get_not_existing_image_with_admin(self):
        response = self.delete(self.admin, image_id=545487654)
        self.assertEqual(response.status_code, 404)

    def test_delete_image_with_contributor(self):
        response = self.delete(self.creator)
        self.assertEqual(response.status_code, 204)

    def test_delete_image_with_some_dude(self):
        response = self.delete(UserF.create())
        self.assertEqual(response.status_code, 404)

    def test_delete_image_with_anonymous(self):
        response = self.delete(AnonymousUser())
        self.assertEqual(response.status_code, 404)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def update_display_field(apps, schema_editor):
    Category = apps.get_model("categories", "Category")
    Field = apps.get_model("categories", "Field")
    for category in Category.objects.all():
        try:
            first_field = category.fields.get(order=0)
            category.display_field = first_field
            category.save()
        except Field.DoesNotExist:
            pass


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0006_category_display_field'),
    ]

    operations = [
        migrations.RunPython(update_display_field),
    ]



from model_utils import Choices

LOCATION_STATUS = Choices('active', 'review')
OBSERVATION_STATUS = Choices(
    'active', 'draft', 'review', 'pending', 'deleted'
)
COMMENT_STATUS = Choices('active', 'deleted')
COMMENT_REVIEW = Choices('open', 'resolved')
MEDIA_STATUS = Choices('active', 'deleted')

ACCEPTED_IMAGE_FORMATS = ('png', 'jpeg', 'gif')
ACCEPTED_VIDEO_FORMATS = ('mpeg', 'mpeg2', 'mp4', 'quicktime', '3gpp', '3gpp2',
                          'avi', 'x-flv', 'x-matroska', 'x-msvideo',
                          'x-ms-wmv', 'ogg', 'webm')



from django.db import models
from django.conf import settings
from django.contrib.gis.db import models as gis
from django.core import mail
from django.template.loader import get_template
from django.template import Context
from django.contrib.sites.models import Site

from .manager import ProjectManager
from .base import STATUS, EVERYONE_CONTRIB


class Project(models.Model):
    """
    Stores a single project.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)
    isprivate = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL)
    everyone_contributes = models.CharField(
        choices=EVERYONE_CONTRIB,
        default=EVERYONE_CONTRIB.auth,
        max_length=20
    )
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )
    admins = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        related_name='admins',
        through='Admins'
    )
    geographic_extend = gis.PolygonField(null=True, geography=True)

    objects = ProjectManager()

    class Meta:
        ordering = ['name']

    def __str__(self):
        return '%s status: %s private: %s' % (
            self.name, self.status, self.isprivate)

    @classmethod
    def create(cls, name, description, isprivate, everyone_contributes,
               creator):
        """
        Creates a new project. Creates two usergroups and adds the creator to
        the administrators user group.

        name : str
            Name of the project
        description : str
            Long-form description of the project
        isprivate : Boolean
            Indicates if the project should be private
        everyone_contributes : str
            Indicates if all you users who have access can contribute to the
            project

            Accepted:
            true - all users who have access, including anonymous
            auth - all user who are authenticated
            false - users need to be member of a user group

        Return
        ------
        geokey.projects.models.Project
        """
        project = cls(
            name=name,
            description=description,
            isprivate=isprivate,
            creator=creator,
            everyone_contributes=everyone_contributes
        )

        project.save()
        Admins.objects.create(project=project, user=creator)

        return project

    def delete(self):
        """
        Deletes the project by setting its status to `DELETED`. Also deletes
        all Admin groups related to the project.
        """
        Admins.objects.filter(project=self).delete()
        self.status = STATUS.deleted
        self.save()

    def re_order_categories(self, order):
        """
        Reorders the categories according to the order given in `order`

        Parameters
        ----------
        order : list
            ordered list of category IDs that define the new order
        """
        categories_to_save = []
        for idx, category_id in enumerate(order):
            category = self.categories.get(pk=category_id)
            category.order = idx
            categories_to_save.append(category)

        for category in categories_to_save:
            category.save()

    def get_role(self, user):
        """
        Returns the user's role in plain text

        Parameters
        ----------
        user : geokey.users.models.User
            User that is examined

        Returns
        -------
        str
            Role of the user in the project
        """
        if self.is_admin(user):
            return 'administrator'
        elif self.can_moderate(user):
            return 'moderator'
        elif self.can_contribute(user):
            return 'contributor'
        else:
            return 'watcher'

    def is_admin(self, user):
        """
        Returns True if the user is member of the administrators group, False
        if not.

        Parameters
        ----------
        user : geokey.users.models.User
            User that is examined

        Returns
        -------
        Boolean
            Indicating if user is admin
        """
        return user in self.admins.all()

    def can_access(self, user):
        """
        Returns True if:
        - the user is member of the administrators group
        - the user is member of one of the usergroups
        - the project is public and has at least one public data grouping

        Parameters
        ----------
        user : geokey.users.models.User
            User that is examined

        Returns
        -------
        Boolean
            Indicating if user is can access
        """

        return self.status == STATUS.active and (self.is_admin(user) or (
            not self.isprivate) or (
            not user.is_anonymous() and (
                self.usergroups.filter(
                    can_contribute=True, users=user).exists() or
                self.usergroups.filter(
                    can_moderate=True, users=user).exists())
            )
        )

    def can_contribute(self, user):
        """
        Returns True if:
        - the user is member of the administrators group
        - the user is member of one usergroup that has can_contribute granted
        - everyone_contributes is True

        Parameters
        ----------
        user : geokey.users.models.User
            User that is examined

        Returns
        -------
        Boolean
            Indicating if user can contribute
        """
        return self.status == STATUS.active and (
            (self.everyone_contributes != 'false' and (
                not user.is_anonymous() or
                not self.everyone_contributes == 'auth')
             ) or self.is_admin(user) or (
                not user.is_anonymous() and (
                    self.usergroups.filter(
                        can_contribute=True, users=user).exists())))

    def can_moderate(self, user):
        """
        Returns True if the user is member of a user group with moderation
        rights

        Parameters
        ----------
        user : geokey.users.models.User
            User that is examined

        Returns
        -------
        Boolean
            Indicating if user can moderate
        """
        return self.status == STATUS.active and (
            self.is_admin(user) or (
                not user.is_anonymous() and (
                    self.usergroups.filter(
                        can_moderate=True, users=user).exists())))

    def is_involved(self, user):
        """
        Returns True if:
        - the user is member of the administrators group
        - the user is member of at least usergroup assigned to the project

        Parameters
        ----------
        user : geokey.users.models.User
            User that is examined

        Returns
        -------
        Boolean
            Indicating if user is involved
        """
        return self.is_admin(user) or (
            not user.is_anonymous() and (
                self.usergroups.filter(users=user).exists()))

    def get_all_contributions(self, user):
        """
        Returns all contributions a user can access in a project. It gets
        the SQL clauses of all data groupings in the project and combines them
        to filter all contributions in the project.

        Parameters
        ----------
        user : geokey.users.models.User
            User that contributions are queried for

        Returns
        -------
        django.db.models.query.QuerySet
            List of geokey.contributions.models.Observations
        """
        data = None
        if self.is_admin(user):
            # Return everything for admins
            return self.observations.for_moderator(user)
        elif self.can_moderate(user):
            data = self.observations.for_moderator(user)
        else:
            data = self.observations.for_viewer(user)

        where_clause = None
        if self.isprivate and not user.is_anonymous():
            clauses = []

            for group in self.usergroups.filter(users=user):
                if group.where_clause is not None:
                    clauses.append(group.where_clause)

            if clauses:
                where_clause = '(' + ') OR ('.join(clauses) + ')'

        if not where_clause:
            return data

        return data.extra(where=[where_clause]).distinct()

    def contact_admins(self, sender, mail_content):
        """
        Sends an email with `mail_content` to all admins of the project, that
        are contact persons.

        Parameters
        ----------
        sender : str
            Email address of the user sending the request
        mail_content : str
            Email text of the request
        """
        messages = []
        email_text = get_template('contact_admins_email.txt')

        platform = Site.objects.get(pk=settings.SITE_ID).name

        for contact_admin in Admins.objects.filter(project=self, contact=True):
            context = Context({
                'sender': sender,
                'admin': contact_admin.user,
                'email_text': mail_content,
                'project_name': self.name,
                'platform': platform
            })
            text = email_text.render(context)

            email = mail.EmailMessage(
                'Enquiry from %s' % sender.display_name,
                text,
                sender.email,
                [contact_admin.user.email]
            )
            messages.append(email)

        if len(messages) > 0:
            connection = mail.get_connection()
            connection.open()
            connection.send_messages(messages)
            connection.close()


class Admins(models.Model):
    """
    An Administator group for a project. Represents the relation between
    Project and User.
    """
    project = models.ForeignKey('Project', related_name='admin_of')
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='has_admins'
    )
    contact = models.BooleanField(default=True)

    class Meta:
        ordering = ['project__name']
        unique_together = ('project', 'user')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
from geokey.contributions.models import Observation


def populate_display_field(apps, schema_editor):
    Category = apps.get_model("categories", "Category")
    # Observation = apps.get_model("contributions", "Observation")

    for category in Category.objects.all():
        if category.display_field is not None:
            for obs in Observation.objects.filter(category=category):
                obs.update_display_field()
                obs.save()


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0012_auto_20150223_1311'),
    ]

    operations = [
        migrations.RunPython(populate_display_field),
    ]



from django.views.generic import CreateView, TemplateView
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.utils.html import strip_tags

from braces.views import LoginRequiredMixin

from oauth2_provider.models import AccessToken

from geokey.core.decorators import handle_exceptions_for_admin

from .forms import AppCreateForm
from .models import Application


class ApplicationOverview(LoginRequiredMixin, TemplateView):
    """
    Displays an overview of all apps a developer has registered
    `/admin/apps/`
    """
    template_name = 'applications/application_overview.html'

    @handle_exceptions_for_admin
    def get_context_data(self, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the user's applications to the context.

        Returns
        -------
        dict
            context
        """
        context = super(ApplicationOverview, self).get_context_data(**kwargs)
        context['apps'] = Application.objects.get_list(self.request.user)
        return context


class ApplicationConnected(LoginRequiredMixin, TemplateView):
    """
    Displays an overview of all apps a user has connected
    `/admin/apps/connected`
    """
    template_name = 'applications/application_connected.html'

    @handle_exceptions_for_admin
    def get_context_data(self, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        all connected applications to the context.

        Parameters
        ----------
        app_id : int
            ID identifying the the app in the database.

        Returns
        -------
        dict
            context
        """
        tokens = AccessToken.objects.filter(
            user=self.request.user).distinct('application')

        apps = [token.application for token in tokens]
        return super(ApplicationConnected, self).get_context_data(
            connected_apps=apps,
            **kwargs
        )


class ApplicationDisconnect(LoginRequiredMixin, TemplateView):
    """
    Disconnect an app
    `/admin/apps/:app_id/disconnect`
    """
    template_name = 'base.html'

    def get(self, request, app_id):
        """
        Handles the get request of the view. It disconnects the application
        by deleting all AccessTokens registered for the app and users.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        app_id : int
            ID identifying the the app in the database.

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to connected apps overview.
        """
        app = Application.objects.get(pk=app_id)

        tokens = AccessToken.objects.filter(
            user=self.request.user,
            application=app
        )
        if tokens:
            tokens.delete()
            messages.success(self.request, "The connection has been deleted.")

        url = reverse('admin:app_connected')
        return HttpResponseRedirect(url)


class ApplicationCreate(LoginRequiredMixin, CreateView):
    """
    Displays the Create Application page.
    `/admin/apps/register`
    """
    form_class = AppCreateForm
    template_name = 'applications/application_create.html'

    def get_success_url(self):
        """
        Returns the redirect url to be called after successful app registering
        """
        return reverse(
            'admin:app_settings',
            kwargs={'app_id': self.object.id}
        )

    def form_valid(self, form):
        """
        Overwrites CreateView method to add the user to the form instance

        Parameters
        ----------
        form : geokey.applications.forms.AppCreateForm
            Represents the user input
        """
        form.instance.user = self.request.user
        messages.success(self.request, "The application has been created.")
        return super(ApplicationCreate, self).form_valid(form)


class ApplicationSettings(LoginRequiredMixin, TemplateView):
    """
    Displays the Application Settings page.
    `/admin/apps/settings`
    """
    template_name = 'applications/application_settings.html'

    @handle_exceptions_for_admin
    def get_context_data(self, app_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the application to the context.

        Parameters
        ----------
        app_id : int
            ID identifying the the app in the database.

        Returns
        -------
        dict
            context
        """
        app = Application.objects.as_owner(self.request.user, app_id)
        users = AccessToken.objects.values('user').filter(
            application=app).distinct().count()
        return super(ApplicationSettings, self).get_context_data(
            application=app, users=users, **kwargs)

    def post(self, request, app_id):
        """
        Handles the POST request and updates the application

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        app_id : int
            ID identifying the the app in the database.

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(app_id)
        app = context.pop('application', None)

        if (app is not None):
            data = request.POST
            app.name = strip_tags(data.get('name'))
            app.description = strip_tags(data.get('description'))
            app.download_url = data.get('download_url')
            app.redirect_uris = data.get('redirect_uris')
            app.authorization_grant_type = data.get('authorization_grant_type')
            app.save()

            messages.success(self.request, "The application has been updated.")
            context['application'] = app

        return self.render_to_response(context)


class ApplicationDelete(LoginRequiredMixin, TemplateView):
    """
    Deletes an app
    `/admin/apps/:app_id/delete`
    """
    template_name = 'base.html'

    @handle_exceptions_for_admin
    def get_context_data(self, app_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the application to the context.

        Parameters
        ----------
        app_id : int
            ID identifying the the app in the database.

        Returns
        -------
        dict
            context
        """
        app = Application.objects.as_owner(self.request.user, app_id)
        return super(ApplicationDelete, self).get_context_data(
            application=app, **kwargs)

    def get(self, request, app_id):
        """
        Handles the GET request and deletes the application

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        app_id : int
            ID identifying the the app in the database.

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to list of apps overview.

        django.http.HttpResponse
            If user is not owner of the app, the error message is rendered.
        """
        context = self.get_context_data(app_id)
        app = context.pop('application', None)

        if app is not None:
            app.delete()

            messages.success(self.request, "The application has been deleted.")
            url = reverse('admin:app_overview')
            return HttpResponseRedirect(url)

        return self.render_to_response(context)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def populate_display_field(apps, schema_editor):
    Observation = apps.get_model("contributions", "Observation")
    for observation in Observation.objects.all():
        first_field = observation.category.fields.get(order=0)
        value = observation.attributes.get(first_field.key)
        observation.display_field = '%s:%s' % (first_field.key, value)
        observation.save()


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0003_auto_20150121_1544'),
    ]

    operations = [
        migrations.RunPython(populate_display_field),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0003_auto_20150123_1148'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='everyone_contributes',
            field=models.CharField(default=b'auth', max_length=20, choices=[(b'true', b'true'), (b'auth', b'auth'), (b'false', b'false')]),
            preserve_default=True,
        ),
    ]



VERSION = (0, 6, 0, 'final', 0)


def get_version():
    version = '%s.%s' % (VERSION[0], VERSION[1])
    if VERSION[2]:
        version = '%s.%s' % (version, VERSION[2])

    sub = ''
    if VERSION[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
        sub = mapping[VERSION[3]] + str(VERSION[4])

    return version + sub



from django import template

from osgeo import ogr

register = template.Library()


@register.filter(name='kml_geom')
def kml_geom(place):
    geojson_geom = place.get('location').get('geometry')
    json_geom = ogr.CreateGeometryFromJson(str(geojson_geom))
    kml_geom = json_geom.ExportToKML()
    return kml_geom


@register.filter(name='kml_name')
def kml_name(place):
    geojson_name = place.get('display_field').get('value')
    return geojson_name


@register.filter(name='kml_desc')
def kml_desc(place):
    geojson_properties = place.get('properties')
    geojson_desc = '<![CDATA[<table>'

    if geojson_properties:
        for key in geojson_properties:
            value = geojson_properties[key]

            if type(value) in [str, unicode]:
                value = value.encode('utf-8')

            if geojson_properties[key] is not None:
                geojson_desc = '{desc}<tr><td>{key}</td><td>{value}</td></tr>'.format(
                    desc=geojson_desc,
                    key=key,
                    value=value
                )

    geojson_desc = geojson_desc + '</table>]]>'

    return geojson_desc


@register.filter(name='kml_style')
def kml_style(place):
    geojson_colour = place.get('meta').get('category').get('colour')
    geojson_colour = geojson_colour.replace('#', '')
    return geojson_colour



import json
from django.test import TestCase

from nose.tools import raises

from geokey.core.exceptions import InputError

from ..models import Field, Category

from .model_factories import (
    TextFieldFactory, NumericFieldFactory, DateTimeFieldFactory,
    LookupFieldFactory, LookupValueFactory, TimeFieldFactory,
    FieldFactory, CategoryFactory, MultipleLookupFieldFactory,
    MultipleLookupValueFactory, DateFieldFactory
)

from geokey.projects.tests.model_factories import ProjectF
from geokey.users.tests.model_factories import UserGroupF
from geokey.contributions.tests.model_factories import ObservationFactory
from geokey.contributions.models import Observation
from geokey.users.models import UserGroup


class CategoryTest(TestCase):
    def test_re_order_fields(self):
        category = CategoryFactory.create()

        field_0 = TextFieldFactory.create(**{'category': category})
        field_1 = TextFieldFactory.create(**{'category': category})
        field_2 = TextFieldFactory.create(**{'category': category})
        field_3 = TextFieldFactory.create(**{'category': category})
        field_4 = TextFieldFactory.create(**{'category': category})

        category.re_order_fields(
            [field_4.id, field_0.id, field_2.id, field_1.id,  field_3.id]
        )

        fields = category.fields.all()

        self.assertTrue(fields.ordered)
        self.assertEqual(fields[0], field_4)
        self.assertEqual(fields[1], field_0)
        self.assertEqual(fields[2], field_2)
        self.assertEqual(fields[3], field_1)
        self.assertEqual(fields[4], field_3)

    def test_re_order_fields_with_false_field(self):
        category = CategoryFactory.create()

        field_0 = TextFieldFactory.create(**{'category': category})
        field_1 = TextFieldFactory.create(**{'category': category})
        field_2 = TextFieldFactory.create(**{'category': category})
        field_3 = TextFieldFactory.create(**{'category': category})
        field_4 = TextFieldFactory.create(**{'category': category})

        try:
            category.re_order_fields(
                [field_4.id, field_0.id, field_2.id, field_1.id,  5854]
            )
        except Field.DoesNotExist:
            fields = category.fields.all()

            self.assertTrue(fields.ordered)
            self.assertEqual(fields[0].order, 0)
            self.assertEqual(fields[1].order, 0)
            self.assertEqual(fields[2].order, 0)
            self.assertEqual(fields[3].order, 0)
            self.assertEqual(fields[4].order, 0)

    @raises(Category.DoesNotExist)
    def test_delete(self):
        category = CategoryFactory.create()
        category.delete()
        Category.objects.get(pk=category.id)

    @raises(Category.DoesNotExist, Observation.DoesNotExist)
    def test_delete_with_observation(self):
        category = CategoryFactory.create()
        observation = ObservationFactory.create(**{
            'category': category
        })
        category.delete()

        Category.objects.get(pk=category.id)
        Observation.objects.get(pk=observation.id)

    def test_delete_with_category_filter(self):
        project = ProjectF.create()
        category = CategoryFactory.create(**{'project': project})
        category_2 = CategoryFactory.create(**{'project': project})

        group = UserGroupF.create(
            **{
                'project': project,
                'filters': {category.id: {}, category_2.id: {}}
            }
        )

        category.delete()

        ref = UserGroup.objects.get(pk=group.id)
        self.assertEqual(ref.filters, {str(category_2.id): {}})

    def test_get_query(self):
        category = CategoryFactory.create()
        query = category.get_query({})
        self.assertEqual(query, '((category_id = %s))' % category.id)

        category = CategoryFactory.create()
        query = category.get_query({
            'min_date': '2014-01-05 00:00'
        })
        self.assertEqual(
            query,
            '((category_id = %s) AND (created_at >= to_date(\'2014-'
            '01-05 00:00\', \'YYYY-MM-DD HH24:MI\')))' % category.id
        )

        category = CategoryFactory.create()
        query = category.get_query({
            'max_date': '2014-01-05 00:00'
        })
        self.assertEqual(
            query,
            '((category_id = %s) AND (created_at <= to_date(\'2014-'
            '01-05 00:00\', \'YYYY-MM-DD HH24:MI\')))' % category.id
        )

        category = CategoryFactory.create()
        query = category.get_query({
            'min_date': '2014-01-01 00:00',
            'max_date': '2014-01-05 00:00'
        })
        self.assertEqual(
            query,
            '((category_id = %s) AND (created_at >= to_date(\'2014-'
            '01-01 00:00\', \'YYYY-MM-DD HH24:MI\')) AND (created_at <= to_'
            'date(\'2014-01-05 00:00\', \'YYYY-MM-DD HH24:MI\')))' % category.id
        )

        category = CategoryFactory.create()
        NumericFieldFactory.create(**{'key': 'number', 'category': category})
        query = category.get_query({
            'min_date': '2014-01-01 00:00',
            'max_date': '2014-01-05 00:00',
            'number': {'minval': 20}
        })
        self.assertEqual(
            query,
            "((category_id = %s) AND (created_at >= to_date(\'2014-"
            "01-01 00:00', 'YYYY-MM-DD HH24:MI')) AND (created_at <= to_"
            "date('2014-01-05 00:00', 'YYYY-MM-DD HH24:MI')) AND (cast(prop"
            "erties ->> 'number' as double precision) >= 20))" % category.id
        )


class FieldTest(TestCase):
    @raises(NotImplementedError)
    def test_field_validate_input(self):
        field = FieldFactory()
        field.validate_input('Bla')

    @raises(NotImplementedError)
    def test_field_type_name(self):
        field = FieldFactory()
        field.type_name

    @raises(NotImplementedError)
    def test_field_get_filter(self):
        field = FieldFactory()
        field.get_filter('Bla')

    def test_get_field_types(self):
        field_types = Field.get_field_types()
        self.assertEqual(len(field_types), 7)

    def test_order(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'TextField'
        )
        self.assertEqual(field.order, 0)

        another_field = Field.create(
            'name-2', 'key-2', 'description', False, category,
            'TextField'
        )
        self.assertEqual(another_field.order, 1)

    @raises(Field.DoesNotExist)
    def test_delete(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'TextField'
        )

        f = Field.objects.get(pk=field.id)
        f.delete()

        Field.objects.get(pk=f.id)

    def test_delete_with_field_filter(self):
        project = ProjectF.create()
        category = CategoryFactory.create(**{'project': project})
        field = Field.create(
            'name', 'key', 'description', False, category,
            'TextField'
        )

        group = UserGroupF.create(
            **{
                'project': project,
                'filters': {
                    category.id: {field.key: 'blah'}}
            }
        )

        field.delete()

        ref = UserGroup.objects.get(pk=group.id)
        self.assertEqual(ref.filters, {str(category.id): {}})


class TextFieldTest(TestCase):
    def test_create_textfield(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'TextField'
        )
        self.assertEqual(field.__class__.__name__, 'TextField')

    def test_get_name(self):
        field = TextFieldFactory()
        self.assertEqual(field.type_name, 'Text')
        self.assertEqual(field.fieldtype, 'TextField')

    def test_textfield_validate_input(self):
        textfield = TextFieldFactory()
        try:
            textfield.validate_input('Bla')
        except InputError:
            self.fail('TextField.validate_input() raised InputError '
                      'unexpectedly!')

    def test_textfield_validate_input_maxlength(self):
        textfield = TextFieldFactory(**{'maxlength': 10})
        try:
            textfield.validate_input('BlaBlaBlaB')
        except InputError:
            self.fail('TextField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_textfield_validate_input_maxlength_too_long(self):
        textfield = TextFieldFactory(**{'maxlength': 10})
        textfield.validate_input('BlaBlaBlaBla')

    def test_textfield_validate_required(self):
        textfield = TextFieldFactory.create(**{'required': True})
        try:
            textfield.validate_input('Bla')
        except InputError:
            self.fail('TextField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_textfield_validate_required_empty_string(self):
        textfield = TextFieldFactory.create(**{'required': True})
        textfield.validate_input('')

    def test_textfield_validate_inactive_required_empty_string(self):
        textfield = TextFieldFactory.create(**{
            'required': True,
            'status': 'inactive'}
        )
        try:
            textfield.validate_input('')
        except InputError:
            self.fail('TextField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_textfield_validate_required_none_type(self):
        textfield = TextFieldFactory.create(**{'required': True})
        textfield.validate_input(None)

    def test_textfield_convert_from_String(self):
        textfield = TextFieldFactory()
        self.assertEqual('Bla', textfield.convert_from_string('Bla'))

    def test_get_filter(self):
        textfield = TextFieldFactory(**{'key': 'key'})
        self.assertEqual(
            textfield.get_filter('blah'),
            "((properties ->> 'key') ILIKE '%%blah%%')"
        )


class NumericFieldTest(TestCase):
    def test_create_numericfield(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'NumericField'
        )
        self.assertEqual(field.__class__.__name__, 'NumericField')

    def test_get_name(self):
        field = NumericFieldFactory()
        self.assertEqual(field.type_name, 'Numeric')
        self.assertEqual(field.fieldtype, 'NumericField')

    def test_numericfield_validate_required(self):
        numericfield = NumericFieldFactory.create(**{'required': True})
        try:
            numericfield.validate_input(2)
        except InputError:
            self.fail('NumericField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_numericfield_validate_required_none_type(self):
        numericfield = NumericFieldFactory.create(**{'required': True})
        numericfield.validate_input(None)

    def test_numericfield_validate_not_required_none_type(self):
        numericfield = NumericFieldFactory.create(**{'required': False})
        try:
            numericfield.validate_input(None)
        except InputError:
            self.fail('numericfield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_numericfield_validate_inactive_required_empty_string(self):
        numericfield = NumericFieldFactory.create(**{
            'required': True,
            'status': 'inactive'}
        )
        try:
            numericfield.validate_input(None)
        except InputError:
            self.fail('numericfield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_numericfield_validate_input_number(self):
        numeric_field = NumericFieldFactory()
        try:
            numeric_field.validate_input(158)
        except InputError:
            self.fail('NumericField.validate_input() raised InputError '
                      'unexpectedly!')

    def test_numericfield_validate_input_string_number(self):
        numeric_field = NumericFieldFactory()
        try:
            numeric_field.validate_input('12')
        except InputError:
            self.fail('NumericField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_numericfield_validate_input_string_char(self):
        numeric_field = NumericFieldFactory()
        numeric_field.validate_input('bla')

    # @raises(InputError)
    def test_numericfield_validate_input_empty_string(self):
        numeric_field = NumericFieldFactory()
        numeric_field.validate_input('')

    def test_numericfield_validate_input_minval(self):
        numeric_field = NumericFieldFactory(**{
            'minval': 10
        })
        try:
            numeric_field.validate_input(12)
        except InputError:
            self.fail('NumericField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_numericfield_validate_input_too_small_minval(self):
        numeric_field = NumericFieldFactory(**{
            'minval': 10
        })
        numeric_field.validate_input(5)

    def test_numericfield_validate_input_maxval(self):
        numeric_field = NumericFieldFactory(**{
            'maxval': 20
        })
        try:
            numeric_field.validate_input(12)
        except InputError:
            self.fail('NumericField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_numericfield_validate_input_too_big_maxval(self):
        numeric_field = NumericFieldFactory(**{
            'maxval': 20
        })
        numeric_field.validate_input(21)
        self.assertTrue(numeric_field.validate_input(12))

    def test_numericfield_validate_input_minval_maxval(self):
        numeric_field = NumericFieldFactory(**{
            'minval': 10,
            'maxval': 20
        })
        try:
            numeric_field.validate_input(12)
        except InputError:
            self.fail('NumericField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_numericfield_validate_input_minval_maxval_too_small(self):
        numeric_field = NumericFieldFactory(**{
            'minval': 10,
            'maxval': 20
        })
        numeric_field.validate_input(5)

    @raises(InputError)
    def test_numericfield_validate_input_minval_maxval_too_big(self):
        numeric_field = NumericFieldFactory(**{
            'minval': 10,
            'maxval': 20
        })
        numeric_field.validate_input(21)

    def test_numericfield_convert_from_string(self):
        numeric_field = NumericFieldFactory()
        self.assertEqual(100, numeric_field.convert_from_string('100'))
        self.assertEqual(1.2, numeric_field.convert_from_string('1.2'))

    def test_numericfield_convert_from_empty_string(self):
        numeric_field = NumericFieldFactory()
        self.assertEqual(None, numeric_field.convert_from_string(''))

    def test_get_filter(self):
        numeric_field = NumericFieldFactory(**{'key': 'key'})
        self.assertEqual(
            numeric_field.get_filter({'minval': 10, 'maxval': 20}),
            "(cast(properties ->> 'key' as double precision) >= 10) AND "
            "(cast(properties ->> 'key' as double precision) <= 20)"
        )
        self.assertEqual(
            numeric_field.get_filter({'minval': 10}),
            "(cast(properties ->> 'key' as double precision) >= 10)"
        )
        self.assertEqual(
            numeric_field.get_filter({'maxval': 20}),
            "(cast(properties ->> 'key' as double precision) <= 20)"
        )


class SingleLookupFieldTest(TestCase):
    def test_create_lookupfield(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'LookupField'
        )
        self.assertEqual(field.__class__.__name__, 'LookupField')

    def test_get_name(self):
        field = LookupFieldFactory()
        self.assertEqual(field.type_name, 'Select box')
        self.assertEqual(field.fieldtype, 'LookupField')

    def test_lookupfield_validate_required(self):
        lookup_field = LookupFieldFactory.create(**{'required': True})
        lookup_value = LookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        lookup_field.validate_input(lookup_value.id)

    @raises(InputError)
    def test_lookupfield_validate_required_none_type(self):
        lookup_field = LookupFieldFactory.create(**{'required': True})
        lookup_field.validate_input(None)

    def test_lookupfield_validate_not_required_none_type(self):
        lookup_field = LookupFieldFactory.create(**{'required': False})
        try:
            lookup_field.validate_input(None)
        except InputError:
            self.fail('lookup_field.validate_input() raised InputError '
                      'unexpectedly!')

    def test_lookupfield_validate_inactive_required_empty_string(self):
        lookup_field = LookupFieldFactory.create(**{
            'required': True,
            'status': 'inactive'}
        )
        try:
            lookup_field.validate_input(None)
        except InputError:
            self.fail('LookupField.validate_input() raised InputError '
                      'unexpectedly!')

    def test_lookupfield_validate_input(self):
        lookup_field = LookupFieldFactory()
        lookup_value = LookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        try:
            lookup_field.validate_input(lookup_value.id)
        except InputError:
            self.fail('LookupField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_lookupfield_validate_wrong_input(self):
        lookup_field = LookupFieldFactory()
        LookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        lookup_field.validate_input(781865458)

    @raises(InputError)
    def test_lookupfield_validate_wrong_input_string(self):
        lookup_field = LookupFieldFactory()
        LookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        lookup_field.validate_input('blah')

    def test_lookupfield_convert_from_string(self):
        lookup_field = LookupFieldFactory()
        lookup_value = LookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': lookup_field
        })
        self.assertEqual(
            lookup_value.id,
            lookup_field.convert_from_string(str(lookup_value.id))
        )
        self.assertIsNone(lookup_field.convert_from_string(''))
        self.assertIsNone(lookup_field.convert_from_string(None))

    def test_get_filter(self):
        lookup_field = LookupFieldFactory(**{'key': 'key'})
        self.assertEqual(
            lookup_field.get_filter([1, 2, 3]),
            '((properties ->> \'key\')::int IN (1,2,3))'
        )


class DateTimeFieldTest(TestCase):
    def test_create_datetimefield(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'DateTimeField'
        )
        self.assertEqual(field.__class__.__name__, 'DateTimeField')

    def test_get_name(self):
        field = DateTimeFieldFactory()
        self.assertEqual(field.type_name, 'Date and Time')
        self.assertEqual(field.fieldtype, 'DateTimeField')

    def test_datetimefield_validate_required(self):
        datetimefield = DateTimeFieldFactory.create(**{'required': True})
        try:
            datetimefield.validate_input('2014-01-01')
        except InputError:
            self.fail('datetimefield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_datetimefield_validate_inactive_required_empty_string(self):
        datetimefield = DateTimeFieldFactory.create(**{
            'required': True,
            'status': 'inactive'}
        )
        try:
            datetimefield.validate_input(None)
        except InputError:
            self.fail('datetimefield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_datetimefield_validate_not_required(self):
        datetimefield = DateTimeFieldFactory.create(**{'required': False})
        try:
            datetimefield.validate_input(None)
        except InputError:
            self.fail('datetimefield.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_datetimefield_validate_required_none_type(self):
        datetimefield = DateTimeFieldFactory.create(**{'required': True})
        datetimefield.validate_input(None)

    def test_datetimefield_validate_input(self):
        date_time_field = DateTimeFieldFactory()
        try:
            date_time_field.validate_input('2014-12-01')
        except InputError:
            self.fail('DateTimeField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_datetimefield_validate_false_input(self):
        date_time_field = DateTimeFieldFactory()
        date_time_field.validate_input('2014-15-01')

    def test_get_filter(self):
        date_time_field = DateTimeFieldFactory(**{'key': 'key'})
        self.assertEqual(
            date_time_field.get_filter(
                {'minval': '2014-10-31 13:00', 'maxval': '2015-02-23 12:00'}
            ),
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD HH24:MI\') >= "
            "to_date(\'2014-10-31 13:00\', \'YYYY-MM-DD HH24:MI\')) AND "
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD HH24:MI\') <= "
            "to_date(\'2015-02-23 12:00\', \'YYYY-MM-DD HH24:MI\'))"
        )
        self.assertEqual(
            date_time_field.get_filter({'minval': '2014-10-31 13:00'}),
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD HH24:MI\') >= "
            "to_date(\'2014-10-31 13:00\', \'YYYY-MM-DD HH24:MI\'))"
        )
        self.assertEqual(
            date_time_field.get_filter({'maxval': '2015-02-23 12:00'}),
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD HH24:MI\') <= "
            "to_date(\'2015-02-23 12:00\', \'YYYY-MM-DD HH24:MI\'))"
        )


class TimeFieldTest(TestCase):
    def test_create_datefield(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'TimeField'
        )
        self.assertEqual(field.__class__.__name__, 'TimeField')

    def test_get_name(self):
        field = TimeFieldFactory()
        self.assertEqual(field.type_name, 'Time')
        self.assertEqual(field.fieldtype, 'TimeField')

    def test_timefield_validate_required(self):
        timefield = TimeFieldFactory.create(**{'required': True})
        try:
            timefield.validate_input('12:45')
        except InputError:
            self.fail('timefield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_timefield_validate_inactive_required_empty_string(self):
        timefield = TimeFieldFactory.create(**{
            'required': True,
            'status': 'inactive'}
        )
        try:
            timefield.validate_input(None)
        except InputError:
            self.fail('DateField.validate_input() raised InputError '
                      'unexpectedly!')

    def test_timefield_validate_not_required(self):
        timefield = TimeFieldFactory.create(**{'required': False})
        try:
            timefield.validate_input(None)
        except InputError:
            self.fail('timefield.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_timefield_validate_required_none_type(self):
        timefield = TimeFieldFactory.create(**{'required': True})
        timefield.validate_input(None)

    def test_timefield_validate_input(self):
        time_field = TimeFieldFactory()
        try:
            time_field.validate_input('12:45')
        except InputError:
            self.fail('DateTimeField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_timefield_validate_false_input(self):
        time_field = TimeFieldFactory()
        time_field.validate_input('25:54')

    def test_get_filter(self):
        time_field = TimeFieldFactory(**{'key': 'key'})
        self.assertEqual(
            time_field.get_filter({'minval': '8:00', 'maxval': '10:00'}),
            '((properties ->> \'key\')::time >= \'8:00\'::time) AND '
            '((properties ->> \'key\')::time <= \'10:00\'::time)'
        )
        self.assertEqual(
            time_field.get_filter({'minval': '21:00', 'maxval': '3:00'}),
            '((properties ->> \'key\')::time >= \'21:00\'::time) OR '
            '((properties ->> \'key\')::time <= \'3:00\'::time)'
        )
        self.assertEqual(
            time_field.get_filter({'minval': '21:00'}),
            '((properties ->> \'key\')::time >= \'21:00\'::time)'
        )
        self.assertEqual(
            time_field.get_filter({'maxval': '21:00'}),
            '((properties ->> \'key\')::time <= \'21:00\'::time)'
        )


class DateFieldTest(TestCase):
    def test_create_datefield(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'DateField'
        )
        self.assertEqual(field.__class__.__name__, 'DateField')

    def test_get_name(self):
        field = DateFieldFactory()
        self.assertEqual(field.type_name, 'Date')
        self.assertEqual(field.fieldtype, 'DateField')

    def test_datefield_validate_required(self):
        datefield = DateFieldFactory.create(**{'required': True})
        try:
            datefield.validate_input('2014-01-01')
        except InputError:
            self.fail('datefield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_datefield_validate_inactive_required_empty_string(self):
        datefield = DateFieldFactory.create(**{
            'required': True,
            'status': 'inactive'}
        )
        try:
            datefield.validate_input(None)
        except InputError:
            self.fail('DateField.validate_input() raised InputError '
                      'unexpectedly!')

    def test_datefield_validate_not_required(self):
        datefield = DateFieldFactory.create(**{'required': False})
        try:
            datefield.validate_input(None)
        except InputError:
            self.fail('datefield.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_datefield_validate_required_none_type(self):
        datetimefield = DateFieldFactory.create(**{'required': True})
        datetimefield.validate_input(None)

    def test_datefield_validate_input(self):
        date_time_field = DateFieldFactory()
        try:
            date_time_field.validate_input('2014-12-01')
        except InputError:
            self.fail('DateTimeField.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_datefield_validate_false_input(self):
        date_time_field = DateFieldFactory()
        date_time_field.validate_input('2014-15-01')

    def test_get_filter(self):
        date_field = DateFieldFactory(**{'key': 'key'})
        self.assertEqual(
            date_field.get_filter(
                {'minval': '2014-10-31', 'maxval': '2015-02-23'}
            ),
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD\') >= "
            "to_date(\'2014-10-31\', \'YYYY-MM-DD\')) AND "
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD\') <= "
            "to_date(\'2015-02-23\', \'YYYY-MM-DD\'))"
        )
        self.assertEqual(
            date_field.get_filter({'minval': '2014-10-31'}),
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD\') >= "
            "to_date(\'2014-10-31\', \'YYYY-MM-DD\'))"
        )
        self.assertEqual(
            date_field.get_filter({'maxval': '2015-02-23'}),
            "(to_date(properties ->> \'key\', \'YYYY-MM-DD\') <= "
            "to_date(\'2015-02-23\', \'YYYY-MM-DD\'))"
        )


class MultipleLookupTest(TestCase):
    def test_create(self):
        category = CategoryFactory()
        field = Field.create(
            'name', 'key', 'description', False, category,
            'MultipleLookupField'
        )
        self.assertEqual(field.__class__.__name__, 'MultipleLookupField')

    def test_get_name(self):
        field = MultipleLookupFieldFactory.create()
        self.assertEqual(field.type_name, 'Multiple select')
        self.assertEqual(field.fieldtype, 'MultipleLookupField')

    def test_convert_from_string(self):
        field = MultipleLookupFieldFactory.create()

        self.assertEqual(None, field.convert_from_string(''))
        self.assertEqual([1, 2, 3], field.convert_from_string('[1, 2, 3]'))

    def test_validate_required(self):
        field = MultipleLookupFieldFactory.create(**{'required': True})
        lookup_value_1 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        lookup_value_2 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        try:
            field.validate_input(
                json.dumps([lookup_value_1.id, lookup_value_2.id])
            )
        except InputError:
            self.fail('multiplelookupfield.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_validate_required_none_type(self):
        field = MultipleLookupFieldFactory.create(**{'required': True})
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        field.validate_input(None)

    def test_lookupfield_validate_not_required_none_type(self):
        field = MultipleLookupFieldFactory.create(**{'required': False})
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        try:
            field.validate_input(None)
        except InputError:
            self.fail('multiplelookupfield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_lookupfield_validate_required_inactive(self):
        field = MultipleLookupFieldFactory.create(
            **{'required': True, 'status': 'inactive'})
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        try:
            field.validate_input(None)
        except InputError:
            self.fail('multiplelookupfield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_validate_string_input(self):
        field = MultipleLookupFieldFactory.create(
            **{'required': True, 'status': 'inactive'})
        lookup_value_1 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        lookup_value_2 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        try:
            field.validate_input(
                json.dumps([lookup_value_1.id, lookup_value_2.id])
            )
            field.validate_input(json.dumps([lookup_value_1.id]))
        except InputError:
            self.fail('multiplelookupfield.validate_input() raised InputError '
                      'unexpectedly!')

    def test_validate_input(self):
        field = MultipleLookupFieldFactory.create(
            **{'required': True, 'status': 'inactive'})
        lookup_value_1 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        lookup_value_2 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        try:
            field.validate_input([lookup_value_1.id, lookup_value_2.id])
            field.validate_input([lookup_value_1.id])
        except InputError:
            self.fail('multiplelookupfield.validate_input() raised InputError '
                      'unexpectedly!')

    @raises(InputError)
    def test_validate_wrong_input(self):
        field = MultipleLookupFieldFactory.create(
            **{'required': True, 'status': 'inactive'})
        lookup_value_1 = MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        MultipleLookupValueFactory(**{
            'name': 'Ms. Piggy',
            'field': field
        })
        field.validate_input(json.dumps([lookup_value_1.id, 8986552121]))

    def test_get_filter(self):
        lookup_field = MultipleLookupFieldFactory(**{'key': 'key'})
        self.assertEqual(
            lookup_field.get_filter(['1', '2', '3']),
            '(regexp_split_to_array(btrim(properties ->> \'key\', \'[]\'), '
            '\',\')::int[] && ARRAY[1, 2, 3])'
        )



from rest_framework import serializers


class FieldSelectorSerializer(serializers.ModelSerializer):
    """
    Special serializer class. Instances accept a `fields` keyword argument
    to set which fields shall be serialized.
    """
    def __init__(self, *args, **kwargs):
        # Don't pass the 'fields' arg up to the superclass
        fields = kwargs.pop('fields', None)

        # Instantiate the superclass normally
        super(FieldSelectorSerializer, self).__init__(*args, **kwargs)

        if fields:
            # Drop any fields that are not specified in the `fields` argument.
            allowed = set(fields)
            existing = set(self.fields.keys())
            for field_name in existing - allowed:
                self.fields.pop(field_name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0004_auto_20150123_1507'),
    ]

    operations = [
        migrations.RunSQL("DELETE FROM projects_admins WHERE project_id IN (SELECT id FROM projects_project WHERE status = 'deleted');")
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0008_auto_20150312_1508'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalobservation',
            options={'ordering': ('-history_date', '-history_id'), 'get_latest_by': 'history_date', 'verbose_name': 'historical observation'},
        ),
        migrations.AlterField(
            model_name='historicalobservation',
            name='history_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
import django_pgjson.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0005_auto_20150202_1135'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalobservation',
            name='properties',
            field=django_pgjson.fields.JsonBField(default={}),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='observation',
            name='properties',
            field=django_pgjson.fields.JsonBField(default={}),
            preserve_default=True,
        ),
    ]



import factory

from django.utils import timezone

from ..models import User, UserGroup


class UserF(factory.django.DjangoModelFactory):
    class Meta:
        model = User

    display_name = factory.Sequence(lambda n: "display_name%s" % n)
    email = factory.Sequence(lambda n: "email%s@example.com" % n)
    password = ''
    is_active = True
    is_superuser = False

    last_login = timezone.datetime(2000, 1, 1).replace(tzinfo=timezone.utc)
    date_joined = timezone.datetime(1999, 1, 1).replace(
        tzinfo=timezone.utc)

    @classmethod
    def _prepare(cls, create, **kwargs):
        password = kwargs.pop('password', None)
        user = super(UserF, cls)._prepare(create, **kwargs)
        if password:
            user.set_password(password)
            if create:
                user.save()
        return user


class UserGroupF(factory.django.DjangoModelFactory):
    class Meta:
        model = UserGroup

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    project = factory.SubFactory(
        'geokey.projects.tests.model_factories.ProjectF')
    can_contribute = True
    can_moderate = False

    @factory.post_generation
    def add_users(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for user in extracted:
                self.users.add(user)



# https://gist.github.com/barrabinfc/426829

from django import http
from django.db import connection

try:
    import settings
    XS_SHARING_ALLOWED_ORIGINS = settings.XS_SHARING_ALLOWED_ORIGINS
    XS_SHARING_ALLOWED_METHODS = settings.XS_SHARING_ALLOWED_METHODS
    XS_SHARING_ALLOWED_HEADERS = settings.XS_SHARING_ALLOWED_HEADERS
except:
    XS_SHARING_ALLOWED_ORIGINS = '*'
    XS_SHARING_ALLOWED_METHODS = ['POST', 'GET', 'OPTIONS', 'PUT', 'PATCH', 'DELETE']
    XS_SHARING_ALLOWED_HEADERS = ['Authorization', 'Content-Type']


class XsSharing(object):
    """
        This middleware allows cross-domain XHR using the html5 postMessage API.

        Access-Control-Allow-Origin: http://foo.example
        Access-Control-Allow-Methods: POST, GET, OPTIONS, PUT, DELETE
    """
    def process_request(self, request):

        if 'HTTP_ACCESS_CONTROL_REQUEST_METHOD' in request.META:
            response = http.HttpResponse()
            response['Access-Control-Allow-Origin'] = XS_SHARING_ALLOWED_ORIGINS
            response['Access-Control-Allow-Methods'] = ",".join( XS_SHARING_ALLOWED_METHODS )
            response['Access-Control-Allow-Headers'] = ",".join( XS_SHARING_ALLOWED_HEADERS )

            return response

        return None

    def process_response(self, request, response):
        # Avoid unnecessary work
        if response.has_header('Access-Control-Allow-Origin'):
            return response

        response['Access-Control-Allow-Origin'] = XS_SHARING_ALLOWED_ORIGINS
        response['Access-Control-Allow-Methods'] = ",".join( XS_SHARING_ALLOWED_METHODS )
        response['Access-Control-Allow-Headers'] = ",".join( XS_SHARING_ALLOWED_HEADERS )

        return response


class TerminalLogging(object):
    def process_response(self, request, response):
        from sys import stdout
        if stdout.isatty():
            for query in connection.queries :
                print "\033[1;31m[%s]\033[0m \033[1m%s\033[0m" % (query['time'],
                    " ".join(query['sql'].split()))
        return response



from django import forms

from .models import Category, Field


class CategoryCreateForm(forms.ModelForm):
    """
    Validates the inputs against the model definition.
    Used in .views.ObservationTypeAdminCreateView
    """

    class Meta:
        model = Category
        fields = ('name', 'description', 'default_status')


class FieldCreateForm(forms.ModelForm):
    """
    Validates the inputs against the model definition.
    Used in .views.FieldAdminCreateView
    """
    class Meta:
        model = Field
        fields = ('name', 'description', 'required')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0003_auto_20150611_1307'),
    ]

    operations = [
        migrations.RunSQL('DROP TABLE IF EXISTS users_groupingusergroup;')
    ]



import json
from rest_framework.renderers import BaseRenderer


class GeoJsonRenderer(BaseRenderer):
    """
    Renderes serialised Contributions into GeoJson
    """
    media_type = 'application/json'
    format = 'json'
    separators = (',', ':')

    def render_single(self, data):
        """
        Renders a single `Contribution` into GeoJson
        """
        data['type'] = "Feature"
        data['geometry'] = json.loads(data.get('location').pop('geometry'))
        return data

    def render_many(self, data):
        """
        Creates a `FeatureCollection` object and adds Contributions to
        `features`.
        """
        return {
            "type": "FeatureCollection",
            "features": [self.render_single(item) for item in data]
        }

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Renders `data` into serialized GeoJson.
        """
        if data is None:
            return ''

        if 'error' in data:
            rendered = data
        elif isinstance(data, dict):
            rendered = self.render_single(data)
        else:
            rendered = self.render_many(data)

        return json.dumps(rendered, separators=self.separators)



from django.core.exceptions import PermissionDenied

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from geokey.core.decorators import handle_exceptions_for_ajax
from geokey.users.models import User
from geokey.projects.models import Project

from ..renderer.geojson import GeoJsonRenderer
from ..parsers.geojson import GeoJsonParser

from .base import SingleAllContribution
from ..serializers import ContributionSerializer


class GeoJsonView(APIView):
    renderer_classes = (GeoJsonRenderer,)
    parser_classes = (GeoJsonParser,)


class ProjectObservations(GeoJsonView):
    """
    Public API endpoint to add new contributions to a project
    /api/projects/:project_id/contributions
    """
    @handle_exceptions_for_ajax
    def post(self, request, project_id):
        """
        Adds a new contribution to a project

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised contribution
        """
        user = request.user
        if user.is_anonymous():
            user = User.objects.get(display_name='AnonymousUser')

        data = request.DATA
        project = Project.objects.as_contributor(request.user, project_id)

        if (not data.get('meta').get('status') == 'draft' and
                project.can_moderate(user)):
            data['meta']['status'] = 'active'

        serializer = ContributionSerializer(
            data=data, context={'user': user, 'project': project}
        )

        if serializer.is_valid(raise_exception=True):
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)

    @handle_exceptions_for_ajax
    def get(self, request, project_id):
        """
        Returns a list of contributions for a project

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised observations
        """
        project = Project.objects.get_single(request.user, project_id)
        contributions = project.get_all_contributions(request.user)

        serializer = ContributionSerializer(
            contributions,
            many=True,
            context={'user': request.user, 'project': project}
        )
        return Response(serializer.data, status=status.HTTP_200_OK)


class ContributionSearchAPIView(GeoJsonView):
    """
    Public API endpoint to search all contributions in a project
    /api/projects/:project_id/contributions/search/?query={query}
    """
    @handle_exceptions_for_ajax
    def get(self, request, project_id):
        """
        Returns a list of contributions that match the query

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised search results
        """
        q = request.GET.get('query')

        project = Project.objects.get_single(request.user, project_id)
        contributions = project.get_all_contributions(request.user).search(q)

        serializer = ContributionSerializer(
            contributions,
            many=True,
            context={'user': request.user, 'project': project, 'search': q}
        )

        return Response(serializer.data, status=status.HTTP_200_OK)


# ############################################################################
#
# SINGLE CONTRIBUTIONS
#
# ############################################################################


class SingleContributionAPIView(GeoJsonView):
    """
    Abstract APIView for handling requests to single observations
    """
    def get_and_respond(self, request, observation):
        """
        Returns a single contributions

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        observation : geokey.contributions.models.Observation
            Observation to be returned

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised observation
        """
        serializer = ContributionSerializer(
            observation,
            context={'user': request.user, 'project': observation.project}
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    def update_and_respond(self, request, observation):
        """
        Updates and returns a single contributions

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        observation : geokey.contributions.models.Observation
            Observation to be returned

        Returns
        -------
        rest_framework.response.Respone
            Contains the updated serialised observation
        """
        data = request.DATA
        user = request.user
        if user.is_anonymous():
            user = User.objects.get(display_name='AnonymousUser')

        new_status = None
        if data.get('meta') is not None:
            new_status = data.get('meta').get('status')

        user_can_moderate = observation.project.can_moderate(user)
        user_is_owner = (observation.creator == user)
        under_review = observation.comments.filter(
            review_status='open').exists()

        if (new_status is not None and new_status != observation.status):
            if not (
                (new_status == 'pending' and
                    (user_is_owner or user_can_moderate)) or
                (new_status == 'active' and
                    observation.status == 'draft' and user_is_owner) or
                (new_status == 'active' and
                    observation.status == 'pending' and user_can_moderate)):

                raise PermissionDenied('You are not allowed to update the '
                                       'status of the contribution from "%s" '
                                       'to "%s"' % (
                                           observation.status,
                                           new_status
                                       ))

        elif not (user_is_owner or user_can_moderate):
            raise PermissionDenied('You are not allowed to update the'
                                   'contribution')

        if new_status == 'active' and under_review:
            data['meta']['status'] = 'review'

        if ((new_status == 'active' and observation.status == 'draft') and
                not user_can_moderate):
            default_status = observation.category.default_status
            data['meta']['status'] = default_status

        serializer = ContributionSerializer(
            observation,
            data=data,
            context={'user': user, 'project': observation.project}
        )
        if serializer.is_valid(raise_exception=True):
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)

    def delete_and_respond(self, request, observation):
        """
        Deletes a single observation

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        observation : geokey.contributions.models.Observation
            Observation to be deleted

        Returns
        -------
        rest_framework.response.Respone
            Empty response indicating successful delete
        """
        if (observation.creator == request.user or
                observation.project.can_moderate(request.user)):
            observation.delete()
            return Response(status=status.HTTP_204_NO_CONTENT)

        raise PermissionDenied('You are not allowed to delete this'
                               'contribution')


class SingleAllContributionAPIView(
        SingleAllContribution, SingleContributionAPIView):
    """
    Public API endpoint for updating a single observation in a project
    /api/projects/:project_id/observations/:observation_id
    """

    @handle_exceptions_for_ajax
    def get(self, request, project_id, observation_id):
        """
        Returns a single contribution

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised observation
        """
        observation = self.get_object(request.user, project_id, observation_id)
        return self.get_and_respond(request, observation)

    @handle_exceptions_for_ajax
    def patch(self, request, project_id, observation_id):
        """
        Updates and returns a single contribution

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the updated serialised observation
        """
        observation = self.get_object(request.user, project_id, observation_id)
        return self.update_and_respond(request, observation)

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, observation_id):
        """
        Deletes a single contribution

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Empty response indicating successful delete
        """
        observation = self.get_object(request.user, project_id, observation_id)
        return self.delete_and_respond(request, observation)



from django.test import TestCase

from rest_framework.serializers import ValidationError

from .model_factories import UserF
from ..serializers import UserSerializer


class UserSerializerTest(TestCase):
    def test_validate_display_name(self):
        UserF.create(**{'display_name': 'name'})
        user = UserF.create()
        serializer = UserSerializer(user)

        try:
            serializer.validate_display_name('name')
        except ValidationError:
            pass
        else:
            self.fail('validate_display_name did not raise ValidationError')

    def test_validate_email(self):
        UserF.create(**{'email': 'name@example.com'})
        user = UserF.create()
        serializer = UserSerializer(user)

        try:
            serializer.validate_email('name@example.com')
        except ValidationError:
            pass
        else:
            self.fail('validate_email did not raise ValidationError')



from json import loads as json_loads
from django.views.generic import TemplateView, CreateView
from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.utils.html import strip_tags

from braces.views import LoginRequiredMixin

from allauth.account.models import EmailAddress

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status

from geokey.applications.models import Application

from geokey.core.decorators import (
    handle_exceptions_for_ajax, handle_exceptions_for_admin
)
from geokey.projects.models import Project, Admins
from geokey.projects.base import STATUS

from .models import User, UserGroup as UserGroupModel
from .serializers import (UserSerializer, UserGroupSerializer)
from .forms import (
    UsergroupCreateForm,
    CustomPasswordChangeForm,
    UserRegistrationForm
)


# ############################################################################
#
# ADMIN VIEWS
#
# ############################################################################


class Index(TemplateView):
    """
    Displays the splash page. Redirects to dashboard if a user is looged in.
    """
    template_name = 'index.html'

    def get(self, request, *args, **kwargs):
        """
        Renders the page or redirects to dashboard if user is authenticated.

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse
            Rendered template

        django.http.HttpResponseRedirect
            to dashboard, if the user is signed in
        """
        if request.user.is_anonymous():
            return self.render_to_response(self.get_context_data)
        else:
            return redirect('admin:dashboard')


class Dashboard(LoginRequiredMixin, TemplateView):
    """
    Displays the dashboard.
    """
    template_name = 'dashboard.html'

    def get_context_data(self):
        """
        Returns the context to render the view. Overwrites the method to add
        projects, projects status types and extensions to context

        Return
        dict
        """
        projects = Project.objects.get_list(self.request.user)

        from geokey.extensions.base import extensions
        ext = []
        for ext_id in extensions.keys():
            extension = extensions.get(ext_id)

            if extension.get('display_admin') and (
                    not extension.get('superuser') or
                    self.request.user.is_superuser):
                ext.append(extension)

        return {
            'admin_projects': projects.filter(admins=self.request.user),
            'involved_projects': projects.exclude(
                admins=self.request.user).exists(),
            'status_types': STATUS,
            'extensions': ext
        }


class UserGroupList(LoginRequiredMixin, TemplateView):
    """
    Displays a list of all user groups for a project.
    """
    template_name = 'users/usergroup_list.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Overwrites the method to add project to the context

        Parameters
        ----------
        project_id : int
            identifies the project in the data base

        Returns
        -------
        dict
        """
        project = Project.objects.as_admin(self.request.user, project_id)
        return super(UserGroupList, self).get_context_data(project=project)


class UserGroupCreate(LoginRequiredMixin, CreateView):
    """
    Displays the create user group page
    `/admin/projects/:project_id/usergroups/new/`
    """
    template_name = 'users/usergroup_create.html'
    form_class = UsergroupCreateForm

    @handle_exceptions_for_admin
    def get_context_data(self, **kwargs):
        """
        Overwrites the method to add project to the context

        Returns
        -------
        dict
        """
        project_id = self.kwargs['project_id']

        context = super(
            UserGroupCreate, self).get_context_data(**kwargs)

        context['project'] = Project.objects.as_admin(
            self.request.user, project_id
        )
        return context

    def get_success_url(self):
        """
        Returns the redirect URL that is called after the user group has been
        created.

        Returns
        -------
        str
            URL that is called after the Group has been created
        """
        project_id = self.kwargs['project_id']
        return reverse(
            'admin:usergroup_overview',
            kwargs={'project_id': project_id, 'group_id': self.object.id}
        )

    def form_valid(self, form):
        """
        Creates the user group

        Parameter
        ---------
        form : geokey.users.forms.UsergroupCreateForm
            Represents the user input
        """
        project_id = self.kwargs['project_id']
        project = Project.objects.as_admin(self.request.user, project_id)

        form.instance.project = project
        messages.success(self.request, "The user group has been created.")
        return super(UserGroupCreate, self).form_valid(form)

    def post(self, request, project_id, *args, **kwargs):
        """
        Creates the user group

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the data base

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        self.object = None
        context = self.get_context_data(*args, **kwargs)
        project = context.get('project')

        if project is not None:
            return super(UserGroupCreate, self).post(
                request, project_id, *args, **kwargs
            )

        return self.render_to_response(context)


class AdministratorsOverview(LoginRequiredMixin, TemplateView):
    """
    Displays the list of administrators of the project
    `/admin/projects/:project_id/usergroups/:group_id/`
    """
    template_name = 'users/usergroup_admins.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Creates the request context for rendering the page, adds the project
        to the context

        Parameters
        ----------
        project_id : int
            identifies the project in the data base

        Return
        ------
        dict
        """
        project = Project.objects.as_admin(self.request.user, project_id)
        return {'project': project}


class UserGroupMixin(object):
    @handle_exceptions_for_admin
    def get_context_data(self, project_id, group_id):
        """
        Creates the request context for rendering the page, adds the user group
        to the context

        Parameter
        ---------
        project_id : int
            identifies the project in the data base
        group_id : int
            identifies the group in the data base

        Returns
        -------
        dict
        """
        project = Project.objects.as_admin(self.request.user, project_id)
        group = project.usergroups.get(pk=group_id)
        return super(UserGroupMixin, self).get_context_data(
            group=group,
            status_types=STATUS
        )


class UserGroupOverview(LoginRequiredMixin, UserGroupMixin, TemplateView):
    """
    Displays the user group settings page
    `/admin/projects/:project_id/usergroups/:group_id/`
    """
    template_name = 'users/usergroup_overview.html'


class UserGroupSettings(LoginRequiredMixin, UserGroupMixin, TemplateView):
    """
    Displays the user group settings page
    `/admin/projects/:project_id/usergroups/:group_id/settings/`
    """
    template_name = 'users/usergroup_settings.html'

    def post(self, request, project_id, group_id):
        """
        Updates the user group settings

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the data base
        group_id : int
            identifies the group in the data base

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, group_id)
        group = context.pop('group', None)

        if group is not None:
            data = request.POST

            group.name = strip_tags(data.get('name'))
            group.description = strip_tags(data.get('description'))
            group.save()

            messages.success(self.request, "The user group has been updated.")
            context['group'] = group

        return self.render_to_response(context)


class UserGroupData(LoginRequiredMixin, UserGroupMixin, TemplateView):
    template_name = 'users/usergroup_data.html'

    def post(self, request, project_id, group_id):
        context = self.get_context_data(project_id, group_id)
        group = context.pop('group', None)

        if group is not None:
            data = request.POST

            if data['filters'] != '-1':
                if data['permission'] == 'all':
                    group.filters = None
                else:
                    group.filters = json_loads(data['filters'])
            group.save()
            context['group'] = group

        return self.render_to_response(context)


class UserGroupPermissions(LoginRequiredMixin, UserGroupMixin, TemplateView):
    """
    Displays the user group settings page
    `/admin/projects/:project_id/usergroups/:group_id/settings/`
    """
    template_name = 'users/usergroup_permissions.html'


class UserGroupDelete(LoginRequiredMixin, UserGroupMixin, TemplateView):
    """
    Deletes the user group
    """
    template_name = 'base.html'

    def get(self, request, project_id, group_id):
        """
        Deletes the user group

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database
        group_id : int
            identifies the group in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            to the user group list

        django.http.HttpResponse
            If user is not administrator of the project, the error message is
            rendered.
        """

        context = self.get_context_data(project_id, group_id)
        group = context.get('group')

        if group is not None:
            group.delete()
            messages.success(self.request, 'The user group has been deleted.')
            return redirect('admin:usergroup_list', project_id=project_id)

        return self.render_to_response(context)


class UserProfile(LoginRequiredMixin, TemplateView):
    """
    Displays the user profile page
    `/admin/profile`
    """
    template_name = 'users/profile.html'

    def post(self, request):
        """
        Updates the user information

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        user = request.user

        user.display_name = request.POST.get('display_name')
        new_email = request.POST.get('email')

        if user.email != new_email:
            email = EmailAddress.objects.get(user=user, email=user.email)
            email.change(request, new_email, confirm=True)

            user.email = new_email

        user.save()

        context = self.get_context_data()
        messages.success(request, 'The user information has been updated.')
        return self.render_to_response(context)


class UserNotifications(LoginRequiredMixin, TemplateView):
    """
    Displays the notifications settings page
    `/admin/profile/notifications/`
    """
    template_name = 'users/notifications.html'

    @handle_exceptions_for_admin
    def get_context_data(self, **kwargs):
        """
        Returns the context to render the view. Adds projects the user is admin
        of.

        Return
        ------
        dict
        """
        user = self.request.user

        context = super(UserNotifications, self).get_context_data(**kwargs)
        context['admins'] = Admins.objects.filter(user=user)

        return context

    @handle_exceptions_for_admin
    def post(self, request):
        """
        Updates the notification settings

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data()
        data = self.request.POST

        for project in context.get('admins'):
            new_val = data.get(str(project.project.id)) is not None

            if project.contact != new_val:
                project.contact = new_val
                project.save()

        messages.success(request, 'Notifications have been updated.')
        return self.render_to_response(context)


# ############################################################################
#
# AJAX VIEWS
#
# ############################################################################

class QueryUsers(APIView):
    """
    AJAX endpoint for querying a list of users
    `/ajax/users/?query={username}
    """
    def get(self, request):
        """
        Returns a list of users where the display_name matches the query

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request

        Returns
        -------
        rest_framework.response.Response
            Contains the list of users
        """
        q = request.GET.get('query').lower()
        users = User.objects.filter(
            display_name__icontains=q).exclude(pk=1)[:10]

        serializer = UserSerializer(
            users, many=True, fields=('id', 'display_name')
        )
        return Response(serializer.data)


class UserGroup(APIView):
    """
    API Endpoints for a usergroup of a project in the AJAX API.
    /ajax/projects/:project_id/usergroups/:usergroup_id/
    """
    @handle_exceptions_for_ajax
    def put(self, request, project_id, group_id):
        """
        Updates user group information

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request
        project_id : int
            identifies the project in the database
        group_id : int
            identifies the group in the database

        Returns
        -------
        rest_framework.response.Response
            Contains the serialised usergroup or an error message
        """
        project = Project.objects.as_admin(request.user, project_id)
        group = project.usergroups.get(pk=group_id)
        serializer = UserGroupSerializer(
            group, data=request.DATA, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class UserGroupUsers(APIView):
    """
    API Endpoints for users in a usergroup of a project in the AJAX API.
    /ajax/projects/:project_id/usergroups/:usergroup_id/users/
    """

    @handle_exceptions_for_ajax
    def post(self, request, project_id, group_id):
        """
        Adds a user to the usergroup

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request
        project_id : int
            identifies the project in the database
        group_id : int
            identifies the group in the database

        Returns
        -------
        rest_framework.response.Response
            Contains the serialised usergroup or an error message
        """
        project = Project.objects.as_admin(request.user, project_id)
        user_id = request.DATA.get('userId')

        try:
            user = User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return Response(
                'The user you are trying to add to the user group does ' +
                'not exist',
                status=status.HTTP_400_BAD_REQUEST
            )

        group = project.usergroups.get(pk=group_id)
        group.users.add(user)

        serializer = UserGroupSerializer(group)
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class UserGroupSingleUser(APIView):
    """
    API Endpoints for a user in a usergroup of a project in the AJAX API.
    /ajax/projects/:project_id/usergroups/:usergroup_id/users/:user_id
    """

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, group_id, user_id):
        """
        Removes a user from the user group

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request
        project_id : int
            identifies the project in the database
        group_id : int
            identifies the group in the database
        user_id : int
            identifies the user in the database

        Returns
        -------
        rest_framework.response.Response
            Empty response if successful
        """
        project = Project.objects.as_admin(request.user, project_id)
        group = project.usergroups.get(pk=group_id)

        user = group.users.get(pk=user_id)
        group.users.remove(user)
        return Response(status=status.HTTP_204_NO_CONTENT)


# ############################################################################
#
# PUBLIC API VIEWS
#
# ############################################################################


class CreateUserMixin(object):
    """
    Mixin to create a user
    """
    def create_user(self, data):
        """
        Creates a user

        Parameters
        ----------
        data : dict
            User data

        Returns
        -------
        geokey.users.models.User
            The newly created user
        """
        user = User.objects.create_user(
            data.get('email'),
            data.get('display_name'),
            password=data.get('password1')
        )
        user.save()
        return user


class UserAPIView(CreateUserMixin, APIView):
    """
    API endpoint to get, update and create a user
    """
    def get(self, request):
        """
        Returns user information

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request

        Response
        --------
        rest_framework.response.Response
            Containing the user info or an error message if no user is signed
            in
        """
        user = request.user
        if not user.is_anonymous():
            serializer = UserSerializer(user)
            return Response(
                serializer.data,
                status=status.HTTP_200_OK
            )
        else:
            return Response(
                {'error': 'You have to be signed in to get user information'},
                status=status.HTTP_401_UNAUTHORIZED
            )

    def patch(self, request):
        """
        Updates user information

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request

        Response
        --------
        rest_framework.response.Response
            Containing the user info or an error message
        """
        user = request.user

        if user.is_anonymous():
            return Response(
                {'error': 'You have to be signed in to get user information'},
                status=status.HTTP_401_UNAUTHORIZED
            )

        data = request.DATA

        serializer = UserSerializer(user, data=data, partial=True)
        if serializer.is_valid():
            new_mail = data.get('email')

            if new_mail is not None and user.email != new_mail:
                try:
                    email = EmailAddress.objects.get(
                        user=user, email=user.email
                    )
                    email.change(request, new_mail, confirm=True)
                except EmailAddress.DoesNotExist:
                    EmailAddress.objects.create(user=user, email=new_mail)

            serializer.save()

            return Response(
                serializer.data,
                status=status.HTTP_200_OK
            )
        else:
            return Response(
                {'error': serializer.errors},
                status=status.HTTP_400_BAD_REQUEST
            )

    def post(self, request):
        """
        Creates a new user

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request

        Response
        --------
        rest_framework.response.Response
            Containing the user info or an error message
        """
        data = request.DATA
        form = UserRegistrationForm(data)
        client_id = data.pop('client_id', None)

        try:
            Application.objects.get(client_id=client_id)
            if form.is_valid():
                user = self.create_user(data)

                EmailAddress.objects.add_email(
                    request._request,
                    user,
                    user.email,
                    signup=True,
                    confirm=True
                )

                serializer = UserSerializer(user)
                return Response(
                    serializer.data,
                    status=status.HTTP_201_CREATED
                )
            else:
                return Response(
                    {'errors': form.errors},
                    status=status.HTTP_400_BAD_REQUEST
                )
        except Application.DoesNotExist:
            return Response(
                {'errors': {'client': 'Client ID not provided or incorrect.'}},
                status=status.HTTP_400_BAD_REQUEST
            )


class ChangePasswordView(APIView):
    """
    API endpoint to change the password via the client
    """
    def post(self, request):
        """
        Updates the password

        Parameter
        ---------
        request : rest_framework.request.Request
            Represents the HTTP request

        Response
        --------
        rest_framework.response.Response
            Empty response indicating success or error message
        """
        user = request.user
        data = request.DATA

        if not user.is_anonymous():
            form = CustomPasswordChangeForm(user, data)
            if form.is_valid():
                form.save()
                return Response(status=status.HTTP_204_NO_CONTENT)
            else:
                return Response(
                    {'errors': form.errors},
                    status=status.HTTP_400_BAD_REQUEST
                )
        else:
            return Response(
                {'error': 'You have to be signed in to change the password.'},
                status=status.HTTP_401_UNAUTHORIZED
            )



from django.core.management.base import NoArgsCommand

from datetime import datetime, timedelta
from pytz import utc

from allauth.account.models import EmailAddress, EmailConfirmation


class Command(NoArgsCommand):
    def set_user_inactive(self, yesterday):
        outstanding = EmailAddress.objects.filter(verified=False)
        for address in outstanding:
            confirmation = EmailConfirmation.objects.filter(
                email_address=address
            ).latest('id')

            if confirmation.sent < yesterday:
                address.user.is_active = False
                address.user.save()

    def handle(self, *args, **options):
        now = datetime.utcnow() - timedelta(1)
        yesterday = datetime(
            now.year, now.month, now.day, 0, 0, 0).replace(tzinfo=utc)

        self.set_user_inactive(yesterday)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0007_auto_20150312_1249'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalobservation',
            name='attributes',
        ),
        migrations.RemoveField(
            model_name='observation',
            name='attributes',
        ),
    ]



from django.test import TestCase
from geokey.extensions.base import ExtensionExists, register, extensions as ext


class RegisterExtensionTest(TestCase):
    def test(self):
        register('text_ext', 'Test', True)
        extension = ext.get('text_ext')
        self.assertEqual(extension.get('ext_id'), 'text_ext')
        self.assertEqual(extension.get('name'), 'Test')
        self.assertEqual(extension.get('index_url'), 'text_ext:index')
        self.assertTrue(extension.get('display_admin'))

    def test_exiting(self):
        ext['text_ext'] = {
            'ext_id': 'text_ext',
            'name': 'Test',
            'display_admin': True,
            'index_url': 'text_ext:index'
        }
        try:
            register('text_ext', 'Test Name', False)
        except ExtensionExists:
            extension = ext.get('text_ext')
            self.assertEqual(extension.get('ext_id'), 'text_ext')
            self.assertEqual(extension.get('name'), 'Test')
            self.assertEqual(
                extension.get('index_url'),
                'text_ext:index'
            )
            self.assertTrue(extension.get('display_admin'))
        else:
            self.fail('ExtensionExists not raised')



from django.template.loader import render_to_string

from rest_framework.renderers import BaseRenderer


class KmlRenderer(BaseRenderer):
    media_type = 'application/vnd.google-earth.kml+xml'
    format = 'kml'

    def render(self, data, accepted_media_type=None, renderer_context=None):
        rendered = render_to_string(
            'geometries/placemarks.kml',
            {'data': data}
        )

        return rendered



"""
WSGI config for opencomap project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/howto/deployment/wsgi/
"""
import os
from django.core.wsgi import get_wsgi_application

settings_module = 'settings'

# We defer to a DJANGO_SETTINGS_MODULE already in the environment. This breaks
# if running multiple sites in the same mod_wsgi process. To fix this, use
# mod_wsgi daemon mode with each site in its own daemon process, or use
# os.environ["DJANGO_SETTINGS_MODULE"] = "dataService.settings"
# os.environ.setdefault("DJANGO_SETTINGS_MODULE", settings_module)
os.environ["DJANGO_SETTINGS_MODULE"] = settings_module

# This application object is used by any WSGI server configured to use this
# file. This includes Django's development server, if the WSGI_APPLICATION
# setting points here.

application = get_wsgi_application()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
from django.db import migrations


def clean_list(val):
    if val is not None and (isinstance(val, str) or isinstance(val, unicode)):
        return json.loads(val)

    return val


def clean_int(val):
    if val is not None and (isinstance(val, str) or isinstance(val, unicode)):
        return int(val)

    return val


def clean_number(val):
    if val is not None and (isinstance(val, str) or isinstance(val, unicode)):
        try:  # it's an int
            return int(val)
        except ValueError:
            pass

        try:  # it's a float
            return float(val)
        except ValueError:
            pass

    # cannot convert to number, returns string or None
    return val


def clean_values(apps, schema_editor):
    Observation = apps.get_model("contributions", "Observation")
    NumericField = apps.get_model("categories", "NumericField")
    LookupField = apps.get_model("categories", "LookupField")
    MultipleLookupField = apps.get_model("categories", "MultipleLookupField")

    for field in NumericField.objects.all():
        for observation in Observation.objects.filter(category=field.category):
            if observation.properties:
                value = observation.properties.get(field.key)
                if value:
                    observation.properties[field.key] = clean_number(value)
                    observation.save()

    for field in LookupField.objects.all():
        for observation in Observation.objects.filter(category=field.category):
            if observation.properties:
                value = observation.properties.get(field.key)
                if value:
                    observation.properties[field.key] = clean_int(value)
                    observation.save()

    for field in MultipleLookupField.objects.all():
        for observation in Observation.objects.filter(category=field.category):
            if observation.properties:
                value = observation.properties.get(field.key)
                if value:
                    observation.properties[field.key] = clean_list(value)
                    observation.save()


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0009_auto_20150420_1549'),
    ]

    operations = [
        migrations.RunPython(clean_values),
    ]
